/*jslint browser: true, eqeqeq: false, evil: true, css: true, forin: true, fragment: true, nomen: false, onevar: false, plusplus: false, undef: true, white: false */
/*window, PFLEX, escape, unescape*/

// *** Header ***

(function () {

// *** START ***
// *** Make sure PFLEX lib doesn't get loaded multiple times on js ad *** 
if (typeof PFLEX == "undefined" || typeof PFLEX.render != "function") {
// *** START ***

// *** Common base.js***

//console.log("Initializing PFLEX");

var d = document,
    q = window;

PFLEX = {
    create: function() {
        function klass() {
            this.init.apply(this, arguments);      
        }
        if (!klass.prototype.init) {
            klass.prototype.init = function() { };
        }
        klass.prototype.constructor = klass;
        return klass;
    },
    $: function (element) {
        if (PFLEX.isString(element)) {
            element = document.getElementById(element);
        }
        return element;
    },
    $A: function (iterable) {
        if (!iterable) { 
            return []; 
        }
        if (PFLEX.browser.isWebkit) {
            if (!(PFLEX.isFunction(iterable) && iterable == '[object NodeList]') && iterable.toArray) {
                return iterable.toArray();
            }
        } else {
            if (iterable.toArray) {
                return iterable.toArray();
            }
        }
        var length = iterable.length || 0, results = new Array(length);
        while (length--) { 
            results[length] = iterable[length];
        }
        return results;
    },
    strip: function(value) {
        if (PFLEX.isString(value)) {
            return value.replace(/^\s+/, '').replace(/\s+$/, '');
        }
        return value;
    },
    parseInputName: function (n) {
        return (/.+:.+/.test(n)) ? n.split(":")[1] : n;
    },
    getFormElement: function (id) {
        var form = PFLEX.$(id);
        if (form) {
            if (form.tagName.toLowerCase() == "form") { 
                return form; 
            }
            if (form.getElementsByTagName("form").length > 0) {
                return form.getElementsByTagName("form")[0];
            }
        }
        return form;
    },
    isOfferSelected: function (input) {
        return (PFLEX.isOfferElement(input) && input.checked);
    },
    isOfferElement: function (input) {
        return (typeof input != "undefined" && input.type == "checkbox" && /.+:pflexoptin/.test(input.id));
    },
    hasClassName: function(element, className) {
        if (!(element = PFLEX.$(element))) return;
        var elementClassName = element.className;
            return (elementClassName.length > 0 && (elementClassName == className || new RegExp("(^|\\s)" + className + "(\\s|$)").test(elementClassName)));
    },
    addClassName: function(element, className) {
        if (!(element = PFLEX.$(element))) return;
        if (!PFLEX.hasClassName(element, className))
            element.className += (element.className ? ' ' : '') + className;
        return element;
    },
    removeClassName: function(element, className) {
        if (!(element = PFLEX.$(element))) return;
        element.className = PFLEX.strip(element.className.replace(new RegExp("(^|\\s+)" + className + "(\\s+|$)"), ' '));
        return element;
    },
    visible: function(element) {
        return PFLEX.$(element).style.display != 'none';
    },
    renderPixel: function (url, doBust, semaphore) {
        var fn;
        if (!url || url === "") { 
            return; 
        }
        var signal = function() {
            semaphore.signal();
        };
        if (typeof semaphore != "undefined" && typeof semaphore == "object") {
            fn = function() { signal(); return; };
        } else {
            fn = function() { return; };
        }
        if (url.indexOf("http") !== 0) {
            url = "http" + PFLEX.prefix + "://" + url;
        }
        if (doBust) {
            var c = (url.indexOf("?") > -1) ? "&" : "?";
            url += (c + Math.round(Math.random() * 2147483647));
        }
        var p = new Image(1, 1);
        p.onload = fn;
        p.onerror = p.onload;
        p.src = url;
        //PFLEX.log("Writing pixel:", url);
    },
        simpleXhr: function(url,successFn,errorFn){
    	var xhrObj = new XMLHttpRequest(),
    	readystatechangeFn = function(event){
    		if (xhrObj.readyState !== 4){ return true; }
    		if (xhrObj.status === 200){
    			successFn(xhrObj);
    		}else{
    			errorFn(xhrObj);
    		}
    	};
    	
	    xhrObj.open('GET', url, true);
        xhrObj.onreadystatechange = readystatechangeFn;
        xhrObj.send(); 
    	
    },
    submit: function (submitUrl, dealId, formData, customData, semaphore, subSourceId) {
        try {
            if (!/^\d+$/.test(dealId)) {
                return;
            }
        } catch(e) {}
        var data = PFLEX.extend(PFLEX.extend({}, formData), PFLEX.isUndefined(customData) ? {} : customData);
        if (PFLEX.isString(subSourceId)) data["pflexsubsrcid"] = subSourceId; 
        var url = submitUrl + "?pontiflexdealid=" + dealId;
        for (var v in data) {
            url += "&" + encodeURIComponent(v) + "=" + encodeURIComponent(data[v]);
        }
        PFLEX.renderPixel(url, false, semaphore);
    },
    addStylesheet: function (css) {
        if (!css || css === "") { return; }
        var s,h,i,b;
        if (PFLEX.browser.isIe) {
            s = d.createStyleSheet();
            s.cssText = css;
        } else {
            h = d.getElementsByTagName("head")[0];
            if (!h) {
                b = d.getElementsByTagName("body")[0];
                h = d.createElement("head");
                b.parentNode.insertBefore(h, b);
            }
            s = d.createElement("style");
            i = (PFLEX.browser.isWebkit) ? 'innerText' : 'innerHTML';
            s[i] = css;
            h.appendChild(s);
        }
    },
    createElement: function (tag, id, properties, attributes, styles) {
        var elm = null, value;
        if (PFLEX.isString(tag)) {
            elm = document.createElement(tag);
            if(id){
            	elm.id = id; 
            }
            for (value in properties) {
                elm[value] = properties[value]; 
            }            
            for (value in attributes) {
                elm.setAttribute(value, attributes[value]);
            }
            for (value in styles) {
                elm.style[value] = styles[value];
            }
        }
        return elm;
    },
    createEl: function(args){
    	// mk - required by following conditionals
    	if(!args.prop){ args.prop = {}; }
    	if(args.html){  args.prop.innerHTML = args.html; }
    	if(args["class"]){ args.prop["className"] = args["class"]; }
    	
    	return PFLEX.createElement.call(this, args.tag, args.id, args.prop, args.attr, args.style);
    },
    render: function (pid, oid, options) {
        if (!pid) { 
            return; 
        }

        if (!oid) {
            oid = null;
        }
        if (!options) {
            options = {}
        }

        var o = {
            options: options,
            oid: oid
        };

        for (var k in options) {
            var v;
            switch (k) {
            case "trackingFunction":
                var fn;
                try {
                    if (typeof options.trackingFunction === "function") {
                        fn = options.trackingFunction;
                    } else if (typeof options.trackingFunction === "string") {
                        fn = new Function(options.trackingFunction);
                    }
                } catch(e) { }
                if (typeof fn === "function") {
                    PFLEX.ActionHandler.add("doCustomUserTrack", fn);                   
                }     
                break;
            case "trackingURL":
                k = "trackingURL";
                v = PFLEX.cleanUrl(options.trackingURL);
                options.trackingURL = v;
                break;
            case "redirectURL":
                k = "redirectURL";
                v = PFLEX.cleanUrl(options.redirectURL);
                options.redirectURL = v;
                break;
            default:
                v = (options[k]) ? options[k] : null;
            }
            if (!v) { continue; }
            o[k] = v;
        }
        if (!o.data) {
            o.data = (typeof pontiflex_data != "undefined") ? PFLEX.extend({}, pontiflex_data) : {};
        }

        PFLEX.Ads.addPlacement(pid, o);
        PFLEX.Ads.addEvent(pid, o);
    },
    parseOptions: function() {
        return PFLEX.deserialize(location.search.substring(1));
    },
    bind: function(method) {
        if (arguments.length < 3 && PFLEX.isUndefined(arguments[1])) return method;
        var args = Array.prototype.slice.call(arguments), fn = args.shift(), object = args.shift();
        return function() {
            return fn.apply(object, args.concat(Array.prototype.slice.call(arguments)));
        };
    },
    isFunction: function(object) {
        return typeof object == "function";
    },
    isString: function(object) {
        return typeof object == "string";
    },
    isNumber: function(object) {
        return typeof object == "number";
    },
    isUndefined: function(object) {
        return typeof object == "undefined";
    },
    isElement: function(object) {
        return !!(object && object.nodeType == 1);
    },
    isInput: function(object) {
        return (PFLEX.isElement(object) && object.nodeName.toLowerCase() == 'input');
    },
    isArray: function(object) {
        return object != null && typeof object == "object" && 'splice' in object && 'join' in object;
    },
    isEmpty: function(object) {
        var prop;
        if (typeof object === 'object' || typeof object === 'function') {
            for (prop in object) {
                if (object.hasOwnProperty(prop)) { return false; }
            }
        }
        return true;
    },
    isOption: function(object) {
        if (PFLEX.isElement(object)) {
            if (!PFLEX.isUndefined(Object.type)) {
                var type  = object.type.toLowerCase();
                return (object.nodeName.toLowerCase() == 'input' || (type == 'radio' || type == 'checkbox'));
            }
        }
        return false;
    },
    isStringBlank: function(val) {
        return (!val || /^\s*$/.test(val));
    },
    isFormElement: function(object) {
        return (PFLEX.isElement(object) && (object.nodeName.toLowerCase() == 'input' ||
            object.nodeName.toLowerCase() == 'textarea' || object.nodeName.toLowerCase() == 'select'));
    },
    isDateField : function(object) {
        return (PFLEX.isElement(object) && object.getAttribute("pflexFieldType") == "date");
    },
    isDatePartField: function(object) {
        var fieldType;
        if (PFLEX.isElement(object)) {
            fieldType = object.getAttribute("pflexFieldType");
        }
        return (fieldType == "year" || fieldType == "month" || fieldType == "day");
    },
    extend: function(destination, source) {
        for (var property in source)
            destination[property] = source[property];
        return destination;
    },
    cleanUrl: function(url) {
        var v, uri;
        if (url && /^http/.test(url)) {
            if (/^http%3A/.test(url)) {
                uri = decodeURI(url);
            } else {
                uri = url;
            }
            v = uri.replace(/(https{0,1}:)(?!\/\/)/i, "$1\/\/");
        } else {
            v = null;
        }
        return v;
    },
    parseUri: function(str) {
        // parseUri 1.2.2
        // (c) Steven Levithan <stevenlevithan.com>
        // MIT License
        var o = {
            strictMode: false,
            key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"],
            q: {  
                name:   "queryKey",
                parser: /(?:^|&)([^&=]*)=?([^&]*)/g
            },
            parser: {
                strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
                loose:  /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
            }
        },
        m = o.parser[o.strictMode ? "strict" : "loose"].exec(str),
        uri = {},
        i = 14;

        while (i--) uri[o.key[i]] = m[i] || "";

        uri[o.q.name] = {};
        uri[o.key[12]].replace(o.q.parser, function ($0, $1, $2) {
            if ($1) uri[o.q.name][$1] = $2;
        });
        return uri;
    },
    sleep: function (ms) {
        var start = new Date().getTime(), sleeping = true;
        while (sleeping) {
            if ((new Date().getTime() - start) > ms) {
                sleeping = false;
            }
        }
    },
    format: function(text) {
        if (arguments.length <= 1) {
            return text;
        }
        var count = arguments.length - 2;
        for (var token = 0; token <= count; token++) {
            text = text.replace(new RegExp("\\{" + token + "\\}", "gi"), arguments[token + 1]);
        }
        return text;
    },
    formatUrl: function(url, placement) {
        if (placement) {
            var name;
            //replace template variables like $first_name$, $last_name$ ...
            for(name in placement.formData) {
                url = url.replace(new RegExp("\\$" + name + "\\$", "gi"), encodeURIComponent(placement.formData[name]));
            }
            
            //replace template variables from options
            for(name in placement.options) {
                if (PFLEX.isNumber(placement.options[name]) || PFLEX.isString(placement.options[name])) {
                    url = url.replace(new RegExp("\\$pflex_o_" + name + "\\$", "gi"), encodeURIComponent(placement.options[name]));
                }
            };
            
            //replace random
            url = url.replace(new RegExp("\\$pflex_random\\$", "gi"),  Math.floor(Math.random()*1E16));
        }
        return url;
    },
    getFormDate: function(form, dateFieldName) {
        return form[dateFieldName + "_yyyy"].value + form[dateFieldName + "_mm"].value + form[dateFieldName + "_dd"].value;
    },
    setFormDate: function(form, dateFieldName, dateFieldValue) {
        var result = /(\d{4})(\d{2})(\d{2})/.exec(dateFieldValue);
        if (result) {
            form[dateFieldName + "_yyyy"].value = result[1];
            form[dateFieldName + "_mm"].value = result[2];
            form[dateFieldName + "_dd"].value = result[3];
        }
    },
    log: function() {
        if (typeof console != "undefined" && typeof console.log == "function") {
            console.log.apply(console, arguments);
        }
    },


    trim: function(stringToTrim) {
	return stringToTrim.replace(/^\s+|\s+$/g,"");
    },
    ltrim: function(stringToTrim) {
	return stringToTrim.replace(/^\s+/,"");
    },
    rtrim: function(stringToTrim) {
	return stringToTrim.replace(/\s+$/,"");
    },
};

PFLEX.protocol = (d.location.protocol.indexOf("http") === 0) ? d.location.protocol.split(":")[0] : "http";

PFLEX.stats = "@adserverweb_stats_domain@";

PFLEX.prefix = (PFLEX.protocol == "https") ? "s": "";

PFLEX.locale = "en";

PFLEX.defaultLocale = "en";

PFLEX.path = function () {
    if (arguments.length == 2) {
        return arguments[1] + '/';
    }
    if (arguments.length == 3) {
        return arguments[1] + '/' + arguments[2] + '/';
    }
    return "";
};

var nObj_ = this.navigator;
var uaString_ = nObj_.userAgent;

PFLEX.browser = { };
PFLEX.browser.version = nObj_.appVersion;
PFLEX.browser.isOpera = uaString_.indexOf('Opera') === 0;
PFLEX.browser.isIe = !PFLEX.browser.isOpera && uaString_.indexOf('MSIE') != -1;
PFLEX.browser.isWebkit = !PFLEX.browser.isOpera && uaString_.indexOf('WebKit') != -1;
PFLEX.browser.isCssSupportOpacity = 
    function() {
        var testdiv = document.createElement("div");
        testdiv.style.display = "none";
        testdiv.innerHTML = "<a style='opacity:.55;'>a<\/a>";
        
        var testlink = testdiv.getElementsByTagName("a")[0];
        return (testlink && /^0.55$/.test( testlink.style.opacity ));
}();

// *** Native Extensions ***

PFLEX.K = function(x) { return x; };

PFLEX.$break = { };

PFLEX.Enumerable = {
    pflexEach: function (iterator, context) {
        var index = 0;
        iterator = PFLEX.bind(iterator, context);
        try {
          this._pflexEach(function(value) {
            iterator(value, index++);
          });
        } catch (e) {
          if (e != PFLEX.$break) throw e;
        }
        return this;
    },
    pflexAll: function (iterator, context) {
        iterator = iterator ? PFLEX.bind(iterator, context) : PFLEX.K;
        var result = true;
        this.pflexEach(function(value, index) {
            result = result && !!iterator(value, index);
            if (!result) throw PFLEX.$break;
        });
        return result;
    },
    pflexAny: function(iterator, context) {
        iterator = iterator ? PFLEX.bind(iterator, context) : PFLEX.K;
        var result = false;
        this.pflexEach(function(value, index) {
            if (result = !!iterator(value, index)) throw PFLEX.$break;
        });
        return result;
    },
    pflexSelect: function (iterator, context) {
        iterator = PFLEX.bind(iterator, context);
        var results = [];
        this.pflexEach(function(value, index) {
            if (iterator(value, index))
                results.push(value);
        });
        return results;
    }
};

PFLEX.extend(Array.prototype, PFLEX.Enumerable);

PFLEX.extend(Array.prototype, {
  _pflexEach: function(iterator) {
    for (var i = 0, length = this.length; i < length; i++)
      iterator(this[i]);
  }
});

PFLEX.ClassNames = PFLEX.create();

PFLEX.ClassNames.prototype = {
    init: function(element) {
        this.element = PFLEX.$(element);
    },
    _pflexEach: function(iterator) {
        this.element.className.split(/\s+/).pflexSelect(function(name) {
            return name.length > 0;
        })._pflexEach(iterator);
    }
};

PFLEX.extend(PFLEX.ClassNames.prototype, PFLEX.Enumerable);

PFLEX.Cookies = PFLEX.create();

PFLEX.extend(PFLEX.Cookies, {
    exists: function(name) {
        return !PFLEX.isStringBlank(PFLEX.Cookies.get(name));
    },
    get: function(name) {
        var r = null, cookies = PFLEX.Cookies.parse();
        if (typeof name === "string") {
            cookie = (typeof cookies[name] !== "undefined" ) ? cookies[name] : null;
        }
        return cookie;
    },
    set: function(name, value, options) {
        if (typeof options !== "object" || options === null) {
            options = {};
        }
        if (typeof value === "undefined" || value === null || typeof value !== "string") {
            value = "";
            options.expiry = -1;
        }
        var cookieOptions = PFLEX.Cookies.makeOptions(options);
        document.cookie = name + "=" + encodeURIComponent(value) + cookieOptions;
    },
    parse: function() {
        var cookies = {}, i, keyval, key, value,
            cookieArray = document.cookie.split(';');
        for (i = 0; i < cookieArray.length; i++) {
            keyval = cookieArray[i].split('=');
            key = keyval[0].replace(/^\s*/,"").replace(/\s*$/,""); //remove leading/trailing whitespace
            try {
                value = decodeURIComponent(keyval[1]);
            } catch(e) {
                value = keyval[1];
            }
            cookies[key] = value;
        }
        return cookies;
    },
    makeOptions: function(options) {
        if (typeof options !== "object" || options === null ) {
             return "; path=" + path;
        }
        var optionsString = "", expiryDate = new Date(),
            expiry = options.expiry,
            path = options.path,
            domain = options.domain,
            secure = options.secure;
        //add expiration date
        if (typeof expiry === "number" && expiry !== 0 ) { // change minutes to date
            expiryDate.setTime(expiryDate.getTime() + (expiry * 60 * 1000));
        }
        if (typeof expiry === "object" && expiry instanceof Date) {
            optionsString = "; expires=" + expiry.toGMTString();
        }
        //set path
        optionsString += (typeof path === "string" && path !== "") ? "; path=" + path : "; path=/";
        //set domain
        optionsString += (typeof domain === "string" && domain !== "") ? "; domain=" + domain : "";
        //set secure
        optionsString += (secure === true ) ? "; secure" : "";
        return optionsString;
    }
});

PFLEX.Coreg = PFLEX.create();

PFLEX.extend(PFLEX.Coreg, {
    cookieNamePrefix: "PFLEX_AD_CR_",
    elements: ["oid", "optin", "cid", "did", "order"],
    cookieOptions: { path: '/', expiry: 60, domain: "@adserverweb_cookie_domain@" },

    exists: function(pid) {
        return PFLEX.Cookies.exists(PFLEX.Coreg.cookieNamePrefix + pid) ? true : false;
    },
    get: function(pid) {
        var cookie,
            offer,
            i = 0,
            j,
            offerData = {};

        cookie = PFLEX.Cookies.get(PFLEX.Coreg.cookieNamePrefix + pid) || "";

        if (cookie != "") {
            cookie = cookie.split(",");
            for (i; i < cookie.length; i++) {
                offer = cookie[i].split("|");
                offerData[offer[0]]={};
                for (j = 0; j < PFLEX.Coreg.elements.length; j++) {
                    offerData[offer[0]][PFLEX.Coreg.elements[j]] = offer[j];
                }
            }
        }
        return offerData;
    },
    set: function(pid, cookie, expiry) {
        var offer,
            cookieArray = [],
            offerArray, i;
                        
        for (offer in cookie) {
            if (cookie.hasOwnProperty(offer)) {
                offerArray = [offer];
                for (i = 1; i < PFLEX.Coreg.elements.length; i++) {
                    offerArray.push(cookie[offer][PFLEX.Coreg.elements[i]]);
                }
                cookieArray.push(offerArray.join("|"));
            }
        }

        PFLEX.Cookies.set(PFLEX.Coreg.cookieNamePrefix + pid, cookieArray.join(","), PFLEX.Coreg.cookieOptions);
    },
    initOffer: function(pid, offerData) {
        var cookie, 
            offer, 
            copy = {}, 
            optin = 0;

        cookie = PFLEX.Coreg.get(pid);
        
        //remove dupes
        for (offer in cookie) {
            if (cookie.hasOwnProperty(offer)) {
                if (offer != offerData.oid) {
                    copy[offer] = cookie[offer];
                } else {
                    optin = cookie[offer].optin;
                }
            }
        }
    
        //add offer
        copy[offerData.oid] = {
            "optin": optin,
            "cid": offerData.cid,
            "did": offerData.did,
            "order": offerData.order
        };
        PFLEX.Coreg.set(pid, copy);
        
        return (optin == 1);
    },
    toggle: function(pid, oid, isOptin) {
        var cookie = PFLEX.Coreg.get(pid);
        if (cookie.hasOwnProperty(oid)) {
            cookie[oid].optin = (isOptin) ? 1 : 0;
        }
        PFLEX.Coreg.set(pid, cookie);
    },
    submit: function(pid) {
        var cookie,
            offer,
            i,
            placement,
            url;

        // Submit placement and offers only if coreg cookie exists
        if (PFLEX.Coreg.exists(pid)) {
            placement = PFLEX.Ads.placement(pid);
            cookie = PFLEX.Coreg.get(pid);

            for (offer in cookie) {
                if (cookie.hasOwnProperty(offer)) {
                    if (cookie[offer].optin == 1) {
                        PFLEX.submit(
                            placement.submitURL,
                            cookie[offer].did,
                            placement.formData, {
                                "pflexoid": offer, 
                                "pflexcrid": cookie[offer].cid
                            },
                            placement.semaphore,
                            placement.subSourceId
                        );
                    }

                    url = PFLEX.stats + "/imp/offer";
                    url += ("/" + offer);
                    if (PFLEX.isString(cookie[offer].cid)) 
                        url += ("/" + cookie[offer].cid);
                    if (PFLEX.isString(placement.subSourceId)) 
                        url += ("/" + placement.subSourceId);
                    PFLEX.renderPixel(url, true);
                }
            }

            url = PFLEX.stats + "/imp/placement";
            url += ("/" + pid);
            if (PFLEX.isString(placement.subSourceId)) 
                url += ("/" + placement.subSourceId);
            PFLEX.renderPixel(url, true);

            PFLEX.Coreg.set(pid, "", -1);
        }
    }
});

PFLEX.console = {
	log: function(messageToLog) {
		console.log(''+messageToLog);
	}
}



// *** Event ***

PFLEX.EventHandler = PFLEX.create();

PFLEX.EventHandler.prototype = {
    init: function (id) {
        this.options = (typeof options == "object") ? options: {};
        this.elementRegistry = {};
        this.registry = {};
        this.id = id;
    },
    registerElement: function (events, elm, oid) {
        var items = [], 
            i, 
            found = false, 
            arr, 
            index = 0;
        if (PFLEX.isElement(elm)) {
            if (PFLEX.isString(events)) {
                items.push(events);
            } else {
                if (PFLEX.isArray(events)) {
                    items = events.slice();
                }
            }
            
            var element = {
                elm: elm, // Element to validate
                oid: oid  // Offer id, this is used for the offer fields (custom fileds linked to an offer)
            };

            for (i = 0; i < items.length; i++) {
                if (this.elementRegistry.hasOwnProperty(items[i])) {
                    // Prevent element to be added more then once
                    arr = this.elementRegistry[items[i]];
                    for (index in arr) {
                        if (arr.hasOwnProperty(index) && arr[index].elm.id == element.elm.id) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        this.elementRegistry[items[i]].push(element);
                    }
                } else {
                    this.elementRegistry[items[i]] = [element];
                }
            }
        }
    },
    getElements: function (event, oid) {
        var type = PFLEX.isString(event) ? event : event.type, 
            items = [],
            arr,
            index;
        if (this.elementRegistry.hasOwnProperty(type)) {
            arr = this.elementRegistry[type];
            for (index in arr) {
                if (arr.hasOwnProperty(index) && arr[index].oid === oid) {
                    items.push(arr[index]);
                }
            }
        }
        return items;
    },
    fire: function (event, parameters) {
        //PFLEX.log("Fired event: ", event);
        var ret = true, array, method, action, params, i, 
            type = PFLEX.isString(event) ? event : event.type;
        
        if (this.registry.hasOwnProperty(type)) {
            array = this.registry[type];
            for (i = 0; i < array.length; i++) {
                action = array[i];
                
                method = action.method;
                // Attach additional params to the already registred params
                params = PFLEX.extend(action.params, parameters ? parameters : {});
                args = action.args;
                args.push(params);

                //PFLEX.log("Running method: ", method, "Args: ", args);
                if (typeof method === "string") {
                    if (!PFLEX.ActionHandler.get(method).run(this, args)) { ret = false; }
                } else {
                    if (!method.apply(this, args)) { ret = false; }
                }

                if (typeof params.onComplete === "string") {
                    this.fire(params.onComplete);
                }
            }
        }

        return ret;
    },
    register: function (event, method, args, parameters) {
        var params = {}, a, k, arr;

        PFLEX.extend(params, parameters ? parameters : {});
        
        arr = PFLEX.isArray(args) ? [event].concat(args) : 
            [event, PFLEX.isString(args) ? args : null, PFLEX.Ads.placement(this.id)];

        var action = {
            method: method, // Name of the action or custom method
            args: arr, // Action arguments
            params: params // Additional params
        };

        if (this.registry.hasOwnProperty(event)) {
            if (!this.actionExists(this.registry[event], action)) {
                this.registry[event].push(action);
            }
        } else {
            this.registry[event] = [action];
        }
    },
    actionExists: function(actions, action) {
        return false;
    }
};

// *** Common storageutil.js***

PFLEX.StorageUtil = {
	addRegistrationData: function(key, value){
		PFLEX.NativeRegistrationStorage.addRegistrationData(key, value);
	},
	getRegistrationData: function(key){
		return PFLEX.NativeRegistrationStorage.getRegistrationData(key);
	},
	clearRegistrationData: function(){
		PFLEX.NativeRegistrationStorage.clearRegistrationData();
	},
	hasValidRegistrationData: function(){
		return PFLEX.NativeRegistrationStorage.hasValidRegistrationData();
	},
	saveOffersState : function() {
		PFLEX.StateStorage.saveOffersState();
    },
    getOffersState : function() {
		return PFLEX.StateStorage.getOffersState();
    },
    clearOffersState: function() {
		PFLEX.StateStorage.clearOffersState();
    },
    saveRegFormState: function() {
		PFLEX.StateStorage.saveRegFormState();
    },
    getRegFormState: function() {
        return PFLEX.StateStorage.getRegFormState();
    },
    clearRegFormState: function() {
        PFLEX.StateStorage.clearRegFormState();
    },
    savePrivacyUrlsState: function() {
        PFLEX.StateStorage.savePrivacyUrlsState();
    },
    getPrivacyUrlsState: function() {
        return PFLEX.StateStorage.getPrivacyUrlsState();
    },
    clearPrivacyUrlsState: function() {
        PFLEX.StateStorage.clearPrivacyUrlsState();
    },
    saveState: function() {
	PFLEX.StateStorage.saveState();
    },
    
    REG_LAST_VERIFIED_DATE_KEY : "pflex.RegistrationLastVerifiedDate",
    REG_LAST_VERIFIED_THRESHOLD_IN_DAYS : 30,

	getRegLastVerifiedDate: function() {
		return PFLEX.NativeRegistrationStorage.getAdditionalData(this.REG_LAST_VERIFIED_DATE_KEY);
	},
	setRegLastVerifiedDate: function(date) {
		PFLEX.NativeRegistrationStorage.setAdditionalData(this.REG_LAST_VERIFIED_DATE_KEY, date);
	},
	hasVerifiedRegistrationData: function() {
		var lastVerifiedDateStr = ''+this.getRegLastVerifiedDate();
		var lastVerifiedDate = new Date(0);
		//console.log('==============Inside hasVerifiedRegistrationData lastVerifiedDate==========='+lastVerifiedDateStr);
		if(lastVerifiedDateStr == null || typeof lastVerifiedDateStr === 'undefined' || lastVerifiedDateStr === 'undefined') {
			lastVerifiedDate = new Date(0);
		} else {
			lastVerifiedDate = PFLEX.DateUtil.parseTimestamp(lastVerifiedDateStr);
		}
		if(typeof lastVerifiedDate === 'undefined' || lastVerifiedDate === null) {
			lastVerifiedDate = new Date(0); 
		}
		var difference = PFLEX.DateUtil.differenceInDays(PFLEX.DateUtil.getUTCNow(), lastVerifiedDate);
		return difference >= 0 && difference <= this.REG_LAST_VERIFIED_THRESHOLD_IN_DAYS;
	}
};

PFLEX.StateStorage = {

    saveOffersState : function() {
                
		var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
		var jsonStr =  JSON.stringify(placement.getOffers());
		PFLEX.NativeStorage.setItem('offers', jsonStr);		
		
    },
    getOffersState : function() {
    		var offersJsonStr = PFLEX.NativeStorage.getItem('offers');
		if(typeof offersJsonStr === "undefined" || offersJsonStr === null) {
			return null;
		} 

		var offersList = JSON.parse(''+offersJsonStr);
		if(typeof offersList === "undefined" || offersList === null) {
			return null;
		}		

		return offersList;
		
    },
    clearOffersState: function() {  
		PFLEX.NativeStorage.removeItem('offers');	
    },
    saveRegFormState: function() {
		var jsonStr = JSON.stringify(PFLEX.SignUp.getFields());
		PFLEX.NativeStorage.setItem('regFields', jsonStr);		
		
    },
    getRegFormState: function() {
    	var fieldsJsonStr = PFLEX.NativeStorage.getItem('regFields');

		if(typeof fieldsJsonStr === "undefined" || fieldsJsonStr === null) {
			return null;
		} 

		var fieldsList = JSON.parse(''+fieldsJsonStr);
		if(typeof fieldsList === "undefined" || fieldsList === null) {
			return null;
		}		

		return fieldsList;
    },
    clearRegFormState: function() {
		PFLEX.NativeStorage.removeItem('regFields');
    },
    savePrivacyUrlsState: function() {
    },
    getPrivacyUrlsState: function() {
    },
    clearPrivacyUrlsState: function() {
    },
    savePendingOffer: function() {
	var pendingOffer = PFLEX.BaseActions.getPendingOffer();
	var pendingOfferJsonStr = JSON.stringify(pendingOffer);
	PFLEX.NativeStorage.setItem('pendingoffer', pendingOfferJsonStr);
   },
   getPendingOffer: function() {
	var pendingOfferJsonStr = PFLEX.NativeStorage.getItem('pendingoffer');		
	if(typeof pendingOfferJsonStr === "undefined" || pendingOfferJsonStr === null) {
		return null;
	}

	var pendingOfferJson = JSON.parse(pendingOfferJsonStr);
	
	if(typeof pendingOfferJson !== "undefined" || pendingOfferJson !== null) {		
		var offersJsonStr = JSON.stringify(pendingOfferJson.offerjson);
		var selected = pendingOfferJson.selected;
		var posted = pendingOfferJson.posted;

		if(typeof offersJsonStr !== "undefined" && offersJsonStr !== null) { 				
			var offer = new PFLEX.Offer(JSON.parse(offersJsonStr), selected, posted);
			return offer;
		}
	}

	return null;

   },
   clearPendingOffer: function() {
	PFLEX.NativeStorage.removeItem('pendingoffer');
   },
   clearState: function() {
	PFLEX.StateStorage.clearOffersState();
	PFLEX.StateStorage.clearRegFormState();
	PFLEX.StateStorage.clearPrivacyUrlsState();
	PFLEX.PageTransitionState.clearSignupPageType();
	PFLEX.PageTransitionState.clearOffersForMissingInfoPage();
	PFLEX.PageTransitionState.clearPrivacyPageConfig();
	PFLEX.PageTransitionState.clearSavedPage();
    },
    saveState: 	function() {
	PFLEX.PageTransitionState.saveSavedPage();
	var savedPage = PFLEX.BaseActions.getCurrentPage();

	if(savedPage !== "undefined") {
		if(savedPage === "multioffer.html") {
			PFLEX.StateStorage.saveOffersState();
		}
		if(savedPage === "multioffer_signup.html") {
			PFLEX.StateStorage.saveOffersState();
			PFLEX.StateStorage.saveRegFormState();
		}
		if(savedPage === "registration.html") {
			PFLEX.StateStorage.saveRegFormState();
			var signupPageType = PFLEX.SignUp.getPageType();
			if(typeof signupPageType !== "undefined" && signupPageType === "missinginfo") {
				PFLEX.PageTransitionState.setSignupPageType('missinginfo');
			}
		}
		if(savedPage === "privacy.html") {
			PFLEX.StateStorage.savePrivacyUrlsState();
		}
		if(savedPage === "offer_signup.html") {
			PFLEX.StateStorage.saveOffersState();
		}



	}

    }
};



PFLEX.PageTransitionState = {
   	setSignupPageType: function(pageType) {
		PFLEX.NativeStorage.setItem('pageType', pageType);			
	},
   	getSignupPageType: function() {
		return PFLEX.NativeStorage.getItem('pageType');
		
	},
   	clearSignupPageType: function() {
		PFLEX.NativeStorage.removeItem('pageType');
	},
   	setOffersForMissingInfoPage: function(offers) {
		var offersListJsonStr = JSON.stringify(offers);
		PFLEX.NativeStorage.setItem('missingInfoPageOffers', offersListJsonStr);		
	},
   	getOffersForMissingInfoPage: function() {
		var offersListJsonStr = null;
		offersListJsonStr = PFLEX.NativeStorage.getItem('missingInfoPageOffers');
				
		if(typeof offersListJsonStr === "undefined" || offersListJsonStr === null) {
			return null;
		} 

		var offersListJson = JSON.parse(''+offersListJsonStr);
		if(typeof offersListJson === "undefined" || offersListJson === null) {
			return null;
		}

		var offers = [];
		for(var key in offersListJson) {
			var offersJsonStr = JSON.stringify(offersListJson[key].offerjson);
			var selected = offersListJson[key].selected;
			var posted = offersListJson[key].posted; 
                        var signupFieldsJsonStr = JSON.stringify(offersListJson[key].signUpFields);

			if(typeof offersJsonStr !== "undefined" && offersJsonStr !== null) { 
				//console.log('=============Inside getOffersForOfferSignupPage signupFieldsJsonStr='+signupFieldsJsonStr);
				
				var offer = new PFLEX.Offer(JSON.parse(offersJsonStr), selected, posted);
	                        var signupFieldsJsonObj = JSON.parse(''+signupFieldsJsonStr); 
				for(var signupKey in signupFieldsJsonObj) {
					var signupField = offer.signUpFields[signupKey];
					var value = signupFieldsJsonObj[signupKey].value;
					if(value !== null || typeof value !== "undefined" || value == "undefined") {
						signupField.value = value;
					}
				}
				offers.push(offer);
			}
		}		

		return offers;

	},
   	clearOffersForMissingInfoPage: function() {
		PFLEX.NativeStorage.removeItem('missingInfoPageOffers');		
	},
   	setOffersForOfferSignupPage: function(offers) {
		var offersListJsonStr = JSON.stringify(offers);
		//console.log('=============Inside setOffersForOfferSignupPage ' + offersListJsonStr);
		PFLEX.NativeStorage.setItem('offerSignupPageOffers', offersListJsonStr);		
	},
   	getOffersForOfferSignupPage: function() {
		var offersListJsonStr = null;
		offersListJsonStr = PFLEX.NativeStorage.getItem('offerSignupPageOffers');
				
		//console.log('=============Inside getOffersForOfferSignupPage step1');

		if(typeof offersListJsonStr === "undefined" || offersListJsonStr === null) {
			return null;
		} 

		var offersListJson = JSON.parse(''+offersListJsonStr);
		if(typeof offersListJson === "undefined" || offersListJson === null) {
			return null;
		}

		var offers = [];
		for(var key in offersListJson) {
			var offersJsonStr = JSON.stringify(offersListJson[key].offerjson);
			var selected = offersListJson[key].selected; 
			var posted = offersListJson[key].posted;
                        var signupFieldsJsonStr = JSON.stringify(offersListJson[key].signUpFields);

			if(typeof offersJsonStr !== "undefined" && offersJsonStr !== null) { 				
				//console.log('=============Inside getOffersForOfferSignupPage offersJsonStr='+offersJsonStr);
				//console.log('=============Inside getOffersForOfferSignupPage signupFieldsJsonStr='+signupFieldsJsonStr);

				var offer = new PFLEX.Offer(JSON.parse(offersJsonStr), selected, posted);
	                        var signupFieldsJsonObj = JSON.parse(''+signupFieldsJsonStr); 
				for(var signupKey in signupFieldsJsonObj) {
					var signupField = offer.signUpFields[signupKey];					
					var value = signupFieldsJsonObj[signupKey].value;
					var populatedFromDevice = signupFieldsJsonObj[signupKey].populatedFromDevice;
					if(value !== null || typeof value !== "undefined" || value == "undefined") {
						signupField.value = value;
						signupField.populatedFromDevice = populatedFromDevice;
					}
				}
				offers.push(offer);
			}
		}		

		return offers;

	},
   	clearOffersForOfferSignupPage: function() {
		//console.log('=============Inside clearOffersForOfferSignupPage');
		PFLEX.NativeStorage.removeItem('offerSignupPageOffers');		
	},
	savePrivacyPageConfig: function() {		
	    var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();
		if(typeof privacyPageConfig !== "undefined" && privacyPageConfig !== null) {
			var jsonStr = JSON.stringify(privacyPageConfig);
			PFLEX.NativeStorage.setItem('privacyPageConfig', ''+jsonStr);
		}
	},
	getPrivacyPageConfig: function() {
		var jsonStr = PFLEX.NativeStorage.getItem('privacyPageConfig');
		if(typeof jsonStr !== "undefined" && jsonStr !== null) {
			var jsonObj = JSON.parse(''+jsonStr);
			//set privacy urls
			var urls = [];
			var urlsJson = JSON.parse(JSON.stringify(jsonObj.privacyUrls));
			for(var key in urlsJson) {
				var targetUrl = urlsJson[key].url;
				if(typeof targetUrl !== "undefined" && targetUrl !== null) {
					var privacyurl = new PFLEX.PrivacyUrl(urlsJson[key].url, urlsJson[key].selected);
					urls.push(privacyurl);					
				}
			}

			var privacyPageConfig = new PFLEX.PrivacyPageConfig(jsonObj.urlToRender, jsonObj.signUpCase, jsonObj.signUpPageType, urls, 
jsonObj.privacyPageType, jsonObj.pageTitle);

			return privacyPageConfig;
		}	
		return null;

    },
    clearPrivacyPageConfig: function() {
	PFLEX.NativeStorage.removeItem('privacyPageConfig');			
    },
    getSavedPage: function() {
	var savedPage = PFLEX.NativeStorage.getItem('savedPage');
	return savedPage;

    },
    saveSavedPage: function() {
	var savedPage = PFLEX.BaseActions.getCurrentPage();
	PFLEX.NativeStorage.setItem('savedPage', savedPage);
    },
    clearSavedPage: function() {
	PFLEX.NativeStorage.removeItem('savedPage');
    }

};

// *** Common AppInfo.js***
// *** <script type="text/javascript" src="./json.js"/> ***


PFLEX.AppInfo = {
    jsonObject: {},
    
    initialize: function(jsonstring) {
		//PFLEX.AppInfo.jsonObject[0] = jsonstring;
		//PFLEX.AppInfo.jsonObject[1] = JSON.parse('' + jsonstring);
    	this.jsonObject[0] = jsonstring;
		this.jsonObject[1] = JSON.parse('' + jsonstring);
		//Following logic is to find postalcode and country field.     
        if (this.jsonObject[1].hasOwnProperty("registrationFields")) {
            //find postal code field name
            var regFields = this.jsonObject[1].registrationFields;
            if (regFields != null) {
                for (var key in regFields) {
		    //use fieldType to get country and postalcode field
		    var fieldType = regFields[key].fieldType;
		    if(fieldType) {
			if(fieldType === "postalcode") {
               	            this.postalCodeFieldName = key;
               	        }
			if (fieldType === "country") {
                            this.countryFieldName = key;
                        }
		    } else {		

                    	var validations = regFields[key].validations;
                    	if (validations != null && validations.length > 0) {
                    	    for (var i = 0; i < validations.length; i++) {
                    	        if (validations[i].toLowerCase() == "postalcode") {
                    	            this.postalCodeFieldName = key;
                    	        }
				if (validations[i].toLowerCase() == "country") {
                        	        this.countryFieldName = key;
                            	}
                        	}
                    	}
		    }
                }
            }
        }
	},

    getAppInfoPflxAdUrl: function() {
	var jsonObj = this.jsonObject[1];
	return jsonObj.pflexadsUrl;
    },
    getDisplayNameForFieldNamed: function (key) {
        var jsonObj = this.getField(key);
        return jsonObj.displayName;
    },
    getField: function (key) {
        var fields = this.getRegistrationFields();
        var jsonStr = fields[key];
        return JSON.parse('' + jsonStr);
    },
    getFieldCount: function () {
        var fieldCount = 0;
	var fields = this.getRegistrationFields();
	fieldCount = fields.length;
	return fieldCount;
    },
    getOffer: function () {
	var jsonObjForOffer = this.getRegistrationOffer();
	if(typeof jsonObjForOffer !== "undefined" && jsonObjForOffer !== null) {		
		return new PFLEX.Offer(jsonObjForOffer, "false", "false");
	} else {
		return;
	}
    },
    getOfferString: function (key) {
	var jsonObjForOffer = this.getRegistrationOffer();
	if(typeof jsonObjForOffer === "undefined" || jsonObjForOffer === null)
		return;
	return jsonObjForOffer.key;
    },
    getOrderedFieldNames: function () {
	var result = {};
	var fields = this.getRegistrationFields();
	var i=0;
	for( var key in fields ) {
		result[i] = key;
		i++;
	}
	return result;
    },
    getOrderedFields: function () {	
	var fieldsMap = this.getRegistrationFields();
	var fields = [];
	for( var key in fieldsMap ) {
		var field = PFLEX.extend(fieldsMap[key], { fieldName : key } );
		fields.push(fieldsMap[key]);
	}
	return this.getSortedFields(fields);
    },
	getClusteredFields : function() {
		var regFields = this.getRegistrationFields();
		return this.clusterFields(regFields, [this.getPostalCodeFieldName(), this.getCountryFieldName()]);
	},
	clusterFields : function(fieldsObj, fieldNames) {
		var cluster_field = {};
		var cluster_field_name = '';
		cluster_field["type"] = "composite";
		cluster_field["fields"] = {};
		for (var i = 0; i < fieldNames.length; i++) {
			if (fieldsObj.hasOwnProperty(fieldNames[i])) {
				var field = fieldsObj[fieldNames[i]];
				if(i==0) {
					cluster_field["order"] = field.order;
				}
				cluster_field.fields[fieldNames[i]] = fieldsObj[fieldNames[i]];
				field.order = i+1;
				delete fieldsObj[fieldNames[i]];
				cluster_field_name = cluster_field_name + '_' + fieldNames[i];
			}
		}
		fieldsObj[cluster_field_name] = cluster_field;
		return fieldsObj;
	},
	getClusteredOrderedFields: function () {	
	var fieldsMap = this.getClusteredFields();
	var fields = [];
	for( var key in fieldsMap ) {
		var field = PFLEX.extend(fieldsMap[key], { fieldName : key } );
		fields.push(fieldsMap[key]);
	}
	return this.getSortedFields(fields);
    },
	getSortedFields: function (fields) {
		return fields.sort(function(a, b){
			var compValue = a.order - b.order;
			if(compValue == 0) {
				if(a.type == "composite") {
					if(b.type == "composite") {
						if(Math.floor(Math.random()*2) == 0) return -1;
						else return 1;
					} else {
						return 1;
					}
				} else {
					if(b.type == "composite") {
						return -1;
					}
				}
				if(PFLEX.isString(a.displayName) & PFLEX.isString(b.displayName)) {
					if(a.displayName == b.displayName){
						return 0;
					}
					return (a.displayName < b.displayName) ? -1 : 1;
				}
			}
			return compValue;
		});
	},
	getPostalCodeFieldName: function() {
		return this.postalCodeFieldName;;
	},
	getCountryFieldName: function() {
		return this.countryFieldName;
	},
    getPid: function () {
	var jsonObj = this.jsonObject[1];	
	return jsonObj.pid;
    },
    getRegistrationFields: function () {
	var jsonObj = this.jsonObject[1];	
	var fieldsStr =  jsonObj.registrationFields;
        
	return JSON.parse(JSON.stringify(fieldsStr));
    },
    getRegistrationOffer: function () {
	var jsonObj = this.jsonObject[1];		
	var fieldsStr =  jsonObj.registrationOffer;
	if(typeof fieldsStr === "undefined" || fieldsStr === null || fieldsStr === "")
		return;

	return JSON.parse(JSON.stringify(fieldsStr));
    },
    getSubSourceId: function () {
	var jsonObj = this.jsonObject[1];	
	return jsonObj.subSourceId;
    },
    getValidationsForFieldNamed: function (key) {	
	var result = {};	
	var json = this.getField(key);
	var arraysJson =  JSON.parse('' + json.validations);

	for (var i = 0; i < arraysJson.length; i++) {
		result[i] = arraysJson[i];
	}
	return result;
    },
    getValue: function (key) {
	var jsonObj = this.jsonObject[1];		
	return jsonObj.key;
    },
    getAdPath: function () {
	return "ads";
    },
    getJsonPath: function () {
	return "json";
    },
    getAppInfoSubSourceIdPath: function() {
	return "subSourceId";
    },
    getAdUrl: function (offerscount) {
	var url = this.getAppInfoPflxAdUrl() + "/" + this.getAdPath() + "/" + this.getJsonPath() +"/"+this.getPid();
	url = url + "?"+this.getAppInfoSubSourceIdPath()+"="+this.getSubSourceId();
	url = url + "&"+"numOffers="+offerscount;
	url = url + "&"+"version="+"2";
        return url;
    },
    
    setDOMObject: function (id) {
        if (PFLEX.$(id) == "undefined") { return; }
        this._DOMObjectId = id;
    },

    hide: function () {
        if (!this._DOMObjectId || PFLEX.isUndefined(PFLEX.$(this._DOMObjectId))) { return; }
        PFLEX.$(this._DOMObjectId).style.display = "none";
        this.visible = false;
    }
};
/* StringHelper.js 
 * 
 * The library is to show the static text on the ads as per locale. 
 */
PFLEX.StringHelper = PFLEX.create();

PFLEX.extend(PFLEX.StringHelper, {
    add: function (locale, messages) {
        var lm = {};
        PFLEX.$A(messages).pflexEach(function(value) {
            lm[locale + "-" + value[0]] = value[1];
        });
        PFLEX.extend(PFLEX.StringHelper.messages, lm);
    },
    get: function(message) {
        var name;
        if (message) {
            name = PFLEX.locale + "-" + message;
            if (PFLEX.StringHelper.messages[name]) {
                return PFLEX.StringHelper.messages[name];
            }
	    return PFLEX.StringHelper.messages[PFLEX.defaultLocale + "-" + message];
        }
        return PFLEX.StringHelper.messages['_DEFAULT_'];
    },
    messages: {
        '_DEFAULT_':'This value is required.'
    }
});

PFLEX.StringHelper.add('en', 
    [['pontiflex', 'Pontiflex'],
    ['hello_world', 'Hello World'],
    ['decline_offer_text', 'No thanks'],
    ['decline_registration_text', 'No Thanks'],
    ['decline_multioffer_text', 'No Thanks'],
    ['confirm_offer_text', 'Confirm'],
    ['confirm_registration_text', 'Continue'],
    ['confirm_multioffer_text', 'Sign Up'],
    ['privacy1_text', 'Your information is secure. Read our '],
    ['privacy2_text', 'Privacy Policy.'],
    ['thank_you_for_registering', 'Thank you. Your sign up is complete.'],
    ['return_to_application', 'Return to Application'],
    ['multi_offer_title', 'Exclusive Deals From Our Partners'],
    ['missing_info_title', 'Complete Your Signup'],
	['missing_info_header', 'Please fill out the form below to complete your signup.'],
	['missing_info_decline', 'No thanks'],
	['missing_info_confirm', 'Complete'],
	['privacy_policy', 'Privacy Policy'],
	['other_policies', 'Other Policies'],
	['select', 'Select'],
	['close', 'Close'],
	['pontiflex_privacy_policy', 'Pontiflex Privacy Policy'],
	['registration_is_required', '  Registration is required.  '],
	['no_offers_selected', 'Please select at least one offer to sign up or press No Thanks.'],
	['fields_prepopulated', 'For your convenience, we have filled out the email field from your phone.'],
	['signup_ads_by','Signup Ads by'],
	['powered_by', 'Powered by'],
	['signup_ads_title','Mobile & Online Signup Ads'],
	['no_offers_available','No offers available in your region.'],
	['offer_signup_header','Thanks! You\'re almost done. One or more of your selected offers require more information. See below!'],
	['offer_signup_nofield_headerformat', 'You have successfully signed up for "{0}. Thanks!"'],
	['offer_signup_field_headerformat', '"{0}" requires information below.'],
	['regulatory_phone', 'You may receive a call or text message shortly after signing up.']
]
);

PFLEX.StringHelper.add('es',
    [['hello_world', 'Hola a todos'],
    ['decline_offer_text', 'No, gracias'],
    ['decline_registration_text', 'Saltar'],
    ['confirm_offer_text', 'Confirmar'],
    ['confirm_registration_text', 'Continuar']]
);
/* optionsHelper.js 
 * 
 * Options values
 */
PFLEX.Options = {};

PFLEX.Options.country = {
"AFGHANISTAN":"AF",
"ALAND ISLANDS":"AX",
"ALBANIA":"AL",
"ALGERIA":"DZ",
"AMERICAN SAMOA":"AS",
"ANDORRA":"AD",
"ANGOLA":"AO",
"ANGUILLA":"AI",
"ANTARCTICA":"AQ",
"ANTIGUA AND BARBUDA":"AG",
"ARGENTINA":"AR",
"ARMENIA":"AM",
"ARUBA":"AW",
"AUSTRALIA":"AU",
"AUSTRIA":"AT",
"AZERBAIJAN":"AZ",
"BAHAMAS":"BS",
"BAHRAIN":"BH",
"BANGLADESH":"BD",
"BARBADOS":"BB",
"BELARUS":"BY",
"BELGIUM":"BE",
"BELIZE":"BZ",
"BENIN":"BJ",
"BERMUDA":"BM",
"BHUTAN":"BT",
"BOLIVIA, PLURINATIONAL STATE OF":"BO",
"BONAIRE, SAINT EUSTATIUS AND SABA":"BQ",
"BOSNIA AND HERZEGOVINA":"BA",
"BOTSWANA":"BW",
"BOUVET ISLAND":"BV",
"BRAZIL":"BR",
"BRITISH INDIAN OCEAN TERRITORY":"IO",
"BRUNEI DARUSSALAM":"BN",
"BULGARIA":"BG",
"BURKINA FASO":"BF",
"BURUNDI":"BI",
"CAMBODIA":"KH",
"CAMEROON":"CM",
"CANADA":"CA",
"CAPE VERDE":"CV",
"CAYMAN ISLANDS":"KY",
"CENTRAL AFRICAN REPUBLIC":"CF",
"CHAD":"TD",
"CHILE":"CL",
"CHINA":"CN",
"CHRISTMAS ISLAND":"CX",
"COCOS (KEELING) ISLANDS":"CC",
"COLOMBIA":"CO",
"COMOROS":"KM",
"CONGO":"CG",
"CONGO, THE DEMOCRATIC REPUBLIC OF THE":"CD",
"COOK ISLANDS":"CK",
"COSTA RICA":"CR",
"COTE D'IVOIRE":"CI",
"CROATIA":"HR",
"CUBA":"CU",
"CURACAO":"CW",
"CYPRUS":"CY",
"CZECH REPUBLIC":"CZ",
"DENMARK":"DK",
"DJIBOUTI":"DJ",
"DOMINICA":"DM",
"DOMINICAN REPUBLIC":"DO",
"ECUADOR":"EC",
"EGYPT":"EG",
"EL SALVADOR":"SV",
"EQUATORIAL GUINEA":"GQ",
"ERITREA":"ER",
"ESTONIA":"EE",
"ETHIOPIA":"ET",
"FALKLAND ISLANDS (MALVINAS)":"FK",
"FAROE ISLANDS":"FO",
"FIJI":"FJ",
"FINLAND":"FI",
"FRANCE":"FR",
"FRENCH GUIANA":"GF",
"FRENCH POLYNESIA":"PF",
"FRENCH SOUTHERN TERRITORIES":"TF",
"GABON":"GA",
"GAMBIA":"GM",
"GEORGIA":"GE",
"GERMANY":"DE",
"GHANA":"GH",
"GIBRALTAR":"GI",
"GREECE":"GR",
"GREENLAND":"GL",
"GRENADA":"GD",
"GUADELOUPE":"GP",
"GUAM":"GU",
"GUATEMALA":"GT",
"GUERNSEY":"GG",
"GUINEA":"GN",
"GUINEA-BISSAU":"GW",
"GUYANA":"GY",
"HAITI":"HT",
"HEARD ISLAND AND MCDONALD ISLANDS":"HM",
"HOLY SEE (VATICAN CITY STATE)":"VA",
"HONDURAS":"HN",
"HONG KONG":"HK",
"HUNGARY":"HU",
"ICELAND":"IS",
"INDIA":"IN",
"INDONESIA":"ID",
"IRAN, ISLAMIC REPUBLIC OF":"IR",
"IRAQ":"IQ",
"IRELAND":"IE",
"ISLE OF MAN":"IM",
"ISRAEL":"IL",
"ITALY":"IT",
"JAMAICA":"JM",
"JAPAN":"JP",
"JERSEY":"JE",
"JORDAN":"JO",
"KAZAKHSTAN":"KZ",
"KENYA":"KE",
"KIRIBATI":"KI",
"KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF":"KP",
"KOREA, REPUBLIC OF":"KR",
"KUWAIT":"KW",
"KYRGYZSTAN":"KG",
"LAO PEOPLE'S DEMOCRATIC REPUBLIC":"LA",
"LATVIA":"LV",
"LEBANON":"LB",
"LESOTHO":"LS",
"LIBERIA":"LR",
"LIBYAN ARAB JAMAHIRIYA":"LY",
"LIECHTENSTEIN":"LI",
"LITHUANIA":"LT",
"LUXEMBOURG":"LU",
"MACAO":"MO",
"MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF":"MK",
"MADAGASCAR":"MG",
"MALAWI":"MW",
"MALAYSIA":"MY",
"MALDIVES":"MV",
"MALI":"ML",
"MALTA":"MT",
"MARSHALL ISLANDS":"MH",
"MARTINIQUE":"MQ",
"MAURITANIA":"MR",
"MAURITIUS":"MU",
"MAYOTTE":"YT",
"MEXICO":"MX",
"MICRONESIA, FEDERATED STATES OF":"FM",
"MOLDOVA, REPUBLIC OF":"MD",
"MONACO":"MC",
"MONGOLIA":"MN",
"MONTENEGRO":"ME",
"MONTSERRAT":"MS",
"MOROCCO":"MA",
"MOZAMBIQUE":"MZ",
"MYANMAR":"MM",
"NAMIBIA":"NA",
"NAURU":"NR",
"NEPAL":"NP",
"NETHERLANDS":"NL",
"NEW CALEDONIA":"NC",
"NEW ZEALAND":"NZ",
"NICARAGUA":"NI",
"NIGER":"NE",
"NIGERIA":"NG",
"NIUE":"NU",
"NORFOLK ISLAND":"NF",
"NORTHERN MARIANA ISLANDS":"MP",
"NORWAY":"NO",
"OMAN":"OM",
"PAKISTAN":"PK",
"PALAU":"PW",
"PALESTINIAN TERRITORY, OCCUPIED":"PS",
"PANAMA":"PA",
"PAPUA NEW GUINEA":"PG",
"PARAGUAY":"PY",
"PERU":"PE",
"PHILIPPINES":"PH",
"PITCAIRN":"PN",
"POLAND":"PL",
"PORTUGAL":"PT",
"PUERTO RICO":"PR",
"QATAR":"QA",
"REUNION":"RE",
"ROMANIA":"RO",
"RUSSIAN FEDERATION":"RU",
"RWANDA":"RW",
"SAINT BARTHELEMY":"BL",
"SAINT HELENA, ASCENSION AND TRISTAN DA CUNHA":"SH",
"SAINT KITTS AND NEVIS":"KN",
"SAINT LUCIA":"LC",
"SAINT MARTIN (FRENCH PART)":"MF",
"SAINT PIERRE AND MIQUELON":"PM",
"SAINT VINCENT AND THE GRENADINES":"VC",
"SAMOA":"WS",
"SAN MARINO":"SM",
"SAO TOME AND PRINCIPE":"ST",
"SAUDI ARABIA":"SA",
"SENEGAL":"SN",
"SERBIA":"RS",
"SEYCHELLES":"SC",
"SIERRA LEONE":"SL",
"SINGAPORE":"SG",
"SINT MAARTEN (DUTCH PART)":"SX",
"SLOVAKIA":"SK",
"SLOVENIA":"SI",
"SOLOMON ISLANDS":"SB",
"SOMALIA":"SO",
"SOUTH AFRICA":"ZA",
"SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS":"GS",
"SPAIN":"ES",
"SRI LANKA":"LK",
"SUDAN":"SD",
"SURINAME":"SR",
"SVALBARD AND JAN MAYEN":"SJ",
"SWAZILAND":"SZ",
"SWEDEN":"SE",
"SWITZERLAND":"CH",
"SYRIAN ARAB REPUBLIC":"SY",
"TAIWAN, PROVINCE OF CHINA":"TW",
"TAJIKISTAN":"TJ",
"TANZANIA, UNITED REPUBLIC OF":"TZ",
"THAILAND":"TH",
"TIMOR-LESTE":"TL",
"TOGO":"TG",
"TOKELAU":"TK",
"TONGA":"TO",
"TRINIDAD AND TOBAGO":"TT",
"TUNISIA":"TN",
"TURKEY":"TR",
"TURKMENISTAN":"TM",
"TURKS AND CAICOS ISLANDS":"TC",
"TUVALU":"TV",
"UGANDA":"UG",
"UKRAINE":"UA",
"UNITED ARAB EMIRATES":"AE",
"UNITED KINGDOM":"GB",
"UNITED STATES":"US",
"UNITED STATES MINOR OUTLYING ISLANDS":"UM",
"URUGUAY":"UY",
"UZBEKISTAN":"UZ",
"VANUATU":"VU",
"VENEZUELA, BOLIVARIAN REPUBLIC OF":"VE",
"VIET NAM":"VN",
"VIRGIN ISLANDS, BRITISH":"VG",
"VIRGIN ISLANDS, U.S.":"VI",
"WALLIS AND FUTUNA":"WF",
"WESTERN SAHARA":"EH",
"YEMEN":"YE",
"ZAMBIA":"ZM",
"ZIMBABWE":"ZW"
};

PFLEX.Options.gender = {
"Male":"M",
"Female":"F",
};

PFLEX.Options.IsdCodesList = {
"AF":"+93",
"AX":"+35818",
"AL":"+355",
"DZ":"+213",
"AS":"+684",
"AD":"+376",
"AO":"+244",
"AI":"+264",
"AQ":"+672",
"AG":"+268",
"AR":"+54",
"AM":"+374",
"AW":"+297",
"AU":"+61",
"AT":"+43",
"AZ":"+994",
"BS":"+242",
"BH":"+973",
"BD":"+880",
"BB":"+246",
"BY":"+375",
"BE":"+32",
"BZ":"+501",
"BJ":"+229",
"BM":"+441",
"BT":"+975",
"BO":"+591",
"BQ":"+5997",
"BA":"+387",
"BW":"+267",
"BV":"+47",
"BR":"+55",
"IO":"+246",
"BN":"+673",
"BG":"+359",
"BF":"+226",
"BI":"+257",
"KH":"+855",
"CM":"+237",
"CA":"+1",
"CV":"+238",
"KY":"+345",
"CF":"+236",
"TD":"+235",
"CL":"+56",
"CN":"+86",
"CX":"+1176",
"CC":"+672",
"CO":"+57",
"KM":"+269",
"CG":"+242",
"CD":"+243",
"CK":"+682",
"CR":"+506",
"CI":"+225",
"HR":"+385",
"CU":"+53",
"CW":"+5999",
"CY":"+357",
"CZ":"+420",
"DK":"+45",
"DJ":"+253",
"DM":"+767",
"DO":"+809",
"EC":"+593",
"EG":"+20",
"SV":"+503",
"GQ":"+240",
"ER":"+291",
"EE":"+372",
"ET":"+251",
"FK":"+500",
"FO":"+298",
"FJ":"+679",
"FI":"+358",
"FR":"+33",
"GF":"+594",
"PF":"+689",
"TF":"+33",
"GA":"+241",
"GM":"+220",
"GE":"+995",
"DE":"+49",
"GH":"+233",
"GI":"+350",
"GR":"+30",
"GL":"+299",
"GD":"+473",
"GP":"+590",
"GU":"+671",
"GT":"+502",
"GG":"+441481",
"GN":"+224",
"GW":"+245",
"GY":"+592",
"HT":"+509",
"HM":"+61",
"VA":"+39",
"HN":"+503",
"HK":"+852",
"HU":"+36",
"IS":"+354",
"IN":"+91",
"ID":"+62",
"IR":"+98",
"IQ":"+964",
"IE":"+353",
"IM":"+441624",
"IL":"+972",
"IT":"+39",
"JM":"+876",
"JP":"+81",
"JE":"+441534",
"JO":"+962",
"KZ":"+7",
"KE":"+254",
"KI":"+686",
"KP":"+850",
"KR":"+82",
"KW":"+965",
"KG":"+996",
"LA":"+856",
"LV":"+371",
"LB":"+961",
"LS":"+266",
"LR":"+231",
"LY":"+218",
"LI":"+423",
"LT":"+370",
"LU":"+352",
"MO":"+853",
"MK":"+389",
"MG":"+261",
"MW":"+265",
"MY":"+60",
"MV":"+960",
"ML":"+223",
"MT":"+356",
"MH":"+692",
"MQ":"+596",
"MR":"+222",
"MU":"+230",
"YT":"+269",
"MX":"+52",
"FM":"+691",
"MD":"+373",
"MC":"+377",
"MN":"+976",
"ME":"+382",
"MS":"+664",
"MA":"+212",
"MZ":"+258",
"MM":"+95",
"NA":"+264",
"NR":"+674",
"NP":"+977",
"NL":"+31",
"NC":"+687",
"NZ":"+64",
"NI":"+505",
"NE":"+227",
"NG":"+234",
"NU":"+683",
"NF":"+672",
"MP":"+670",
"NO":"+47",
"OM":"+968",
"PK":"+92",
"PW":"+680",
"PS":"+970",
"PA":"+507",
"PG":"+675",
"PY":"+595",
"PE":"+51",
"PH":"+63",
"PN":"+649",
"PL":"+48",
"PT":"+351",
"PR":"+787",
"QA":"+974",
"RE":"+262",
"RO":"+40",
"RU":"+7",
"RW":"+250",
"BL":"+590",
"SH":"+290",
"KN":"+869",
"LC":"+758",
"MF":"+590",
"PM":"+508",
"VC":"+1809",
"WS":"+684",
"SM":"+378",
"ST":"+239",
"SA":"+966",
"SN":"+221",
"RS":"+381",
"SC":"+248",
"SL":"+232",
"SG":"+65",
"SX":"+1721",
"SK":"+421",
"SI":"+386",
"SB":"+677",
"SO":"+252",
"ZA":"+27",
"GS":"+44",
"ES":"+34",
"LK":"+94",
"SD":"+249",
"SR":"+597",
"SJ":"+47",
"SZ":"+268",
"SE":"+46",
"CH":"+41",
"SY":"+963",
"TW":"+886",
"TJ":"+992",
"TZ":"+255",
"TH":"+66",
"TL":"+220",
"TG":"+228",
"TK":"+690",
"TO":"+676",
"TT":"+868",
"TN":"+216",
"TR":"+90",
"TM":"+993",
"TC":"+649",
"TV":"+688",
"UG":"+256",
"UA":"+380",
"AE":"+971",
"GB":"+44",
"US":"+1",
"UM":"+1",
"UY":"+598",
"UZ":"+998",
"VU":"+678",
"VE":"+58",
"VN":"+84",
"VG":"+1284",
"VI":"+1340",
"WF":"+681",
"EH":"+21228",
"YE":"+967",
"ZM":"+260",
"ZW":"+263"
};

PFLEX.Options.state = {
"US":{
"Alaska":"AK",
"Alabama":"AL",
"Arkansas":"AR",
"American Samoa":"AS",
"Arizona":"AZ",
"California":"CA",
"Colorado":"CO",
"Connecticut":"CT",
"D.C.":"DC",
"Delaware":"DE",
"Florida":"FL",
"Micronesia":"FM",
"Georgia":"GA",
"Guam":"GU",
"Hawaii":"HI",
"Iowa":"IA",
"Idaho":"ID",
"Illinois":"IL",
"Indiana":"IN",
"Kansas":"KS",
"Kentucky":"KY",
"Louisiana":"LA",
"Massachusetts":"MA",
"Maryland":"MD",
"Maine":"ME",
"Marshall Islands":"MH",
"Michigan":"MI",
"Minnesota":"MN",
"Missouri":"MO",
"Marianas":"MP",
"Mississippi":"MS",
"Montana":"MT",
"North Carolina":"NC",
"North Dakota":"ND",
"Nebraska":"NE",
"New Hampshire":"NH",
"New Jersey":"NJ",
"New Mexico":"NM",
"Nevada":"NV",
"New York":"NY",
"Ohio":"OH",
"Oklahoma":"OK",
"Oregon":"OR",
"Pennsylvania":"PA",
"Puerto Rico":"PR",
"Palau":"PW",
"Rhode Island":"RI",
"South Carolina":"SC",
"South Dakota":"SD",
"Tennessee":"TN",
"Texas":"TX",
"Utah":"UT",
"Virginia":"VA",
"Virgin Islands":"VI",
"Vermont":"VT",
"Washington":"WA",
"Wisconsin":"WI",
"West Virginia":"WV",
"Wyoming":"WY",
"Military Americas":"AA",
"Military Europe/ME/Canada":"AE",
"Military Pacific":"AP"
},
"CA":{
"Alberta":"AB",
"Manitoba":"MB",
"Alberta":"AB",
"British Columbia":"BC",
"Manitoba":"MB",
"New Brunswick":"NB",
"Newfoundland and Labrador":"NL",
"Nova Scotia":"NS",
"Northwest Territories":"NT",
"Nunavut":"NU",
"Ontario":"ON",
"Prince Edward Island":"PE",
"Quebec":"QC",
"Saskatchewan":"SK",
"Yukon Territory":"YT"
}
};

PFLEX.Options.getYearsOptions = function(optionType) {
	var result = [];
	var startDate = null;
	var endDate = null;
	var now = new Date();
	if(optionType === 'dateofbirth') {		
		startDate = new Date(now.getFullYear()-100, now.getMonth(), now.getDate());
		endDate = new Date(now.getFullYear()-18, now.getMonth(), now.getDate());

	} else if(optionType === 'childdob') {
		startDate = new Date(now.getFullYear()-2, now.getMonth(), now.getDate());
		endDate = new Date(now.getFullYear(), now.getMonth()+9, now.getDate());
	} else {
		startDate = new Date(now.getFullYear()-100, now.getMonth(), now.getDate());
		endDate = new Date(now.getFullYear()+2, now.getMonth(), now.getDate());
	}
	if(startDate !== null || endDate !== null) {
		for(var i = endDate.getFullYear(); i >= startDate.getFullYear(); i--) {
			result.push(i);
		}		
	}
	return result;


};
// *** dateutil.js Validator ***

PFLEX.DateData = PFLEX.create();

PFLEX.DateData.prototype = {
    init: function (year, month, day) {
        this.year = year;
        this.month = month;
        this.day = day;
    }
};

PFLEX.DateUtil = {
    parseDateFromForm : function(elm) {
        var options = elm.getElementsByTagName('select'), year = -1, month = -1, day = -1;
        PFLEX.$A(options).pflexAny(function (elm) {
            var selected = elm.options ? elm.selectedIndex > 0 : false;
            var fieldType = elm.getAttribute("pflexFieldType");
            if (selected) {
                if (fieldType == "year") {
                    year = elm.value;
                } else if (fieldType == "month") {
                    month = elm.value;
                } else if (fieldType == "day") {
                    day = elm.value;
                }
            }
        });
        return new PFLEX.DateData(year, month, day);
    },
    dateDiff : function(date1, date2) {
        //date1 and date2 are DateData objects - calculates upto month resolution
        var diff = new PFLEX.DateData(date1.year - date2.year, date1.month - date2.month, date1.day - date2.day);
        if (diff.year > 0) {
            if (diff.month < 0) {
                diff.year--;
                diff.month+=12;
            } else if (diff.month == 0) {
                if (diff.day < 0) {
                    diff.year--;
                    diff.month+=12;
                }
            }
            if (diff.month > 0 && diff.day < 0) {
                diff.month--;
            }
        } else if (diff.year == 0) {
            if (diff.month < 0) {
                if (diff.day > 0) {
                    diff.month++;
                }
            } else if (diff.month > 0) {
                if (diff.day < 0) {
                    diff.month--;
                }
            }
        } else if (diff.year < 0) {
            if (diff.month > 0) {
                diff.year++;
                diff.month-=12;
            } else if (diff.month == 0) {
                if (diff.day > 0) {
                    diff.year++;
                    diff.month-=12;
                }
            }
            if (diff.month < 0 && diff.day > 0) {
                diff.month++;
            }
        }
        diff.day = 0; //set day diff to 0 as date diff is only upto month resolution
        //PFLEX.log("Date diff: " + diff.year + ":" + diff.month);
        return diff;
    },
	differenceInDays: function(date1, date2) {
		//returns difference of date1 and date2 in days.
		if(date1 == null || typeof date1 === undefined || date2 == null || typeof date2 === undefined) {
			throw "Invalid dates for computing difference";
		}
		return (date1.getTime() - date2.getTime())/(24*60*60*1000);
	},
	differenceInSeconds: function(date1, date2) {
		//returns difference of date1 and date2 in days.
		if(date1 == null || typeof date1 === undefined || date2 == null || typeof date2 === undefined) {
			throw "Invalid dates for computing difference";
		}
		return (date1.getTime() - date2.getTime())/(1000);
	},
	getUTCNow: function() {
		var d = new Date();
		return new Date(d.getUTCFullYear(), d.getUTCMonth(), d.getUTCDate(),
		d.getUTCHours(),d.getUTCMinutes(),d.getUTCSeconds(),d.getUTCMilliseconds());
	},
	parse: function(dateString) {
		if(dateString == null || typeof dateString === undefined) {
			return null;
		}
		var values = (dateString+'').split('-');
		if(values.length < 6) {
			throw "Invalid date to parse";
		}
		return new Date(Number(values[0]), Number(values[1]), Number(values[2]), Number(values[3]), Number(values[4]), Number(values[5]));
	},
	parseDate: function(dateString) {
		if(dateString == null || typeof dateString === undefined) {
			return null;
		}
		if(dateString.indexOf('-') > 0 || dateString.length < 8) {
			return null;
		}

		return new Date(Number(dateString.substring(0,4)), Number(dateString.substring(4,6))-1, Number(dateString.substring(6,8)));
	},
	parseTimestamp: function(dateString) {
		if(dateString == null || typeof dateString === undefined) {
			return null;
		}
		if(dateString.indexOf('-') > 0 || dateString.length < 14) {
			return null;
		}
		return new Date(Number(dateString.substring(0,4)), Number(dateString.substring(4,6))-1, Number(dateString.substring(6,8)), Number(dateString.substring(8,10)), Number(dateString.substring(10,12)), Number(dateString.substring(12,14)));
	},
	format: function(date) {
		if(date == null || typeof date === undefined) {
			return null;
		}
		return ''+date.getFullYear()+'-'+date.getMonth()+'-'+
		date.getDate()+'-'+date.getHours()+'-'+date.getMinutes()+'-'+date.getSeconds();
	},
	formatAsDate: function(date) {
		var X;
    		with (date) {
		        X = String((getFullYear() * 100 + getMonth() + 1) * 100 + getDate());
    		}
		return ''+X;
	},
	formatAsTimestamp: function(date) {
		var X;
    		with (date) {
		        X = ((((getFullYear() * 100 + getMonth() + 1) * 100 + getDate()) * 100 + getHours()) * 100 + getMinutes()) * 100 + getSeconds();
    		}
		return ''+X;
	},
    addToDate: function(date1, time, timeUnit) {
        if (date1 && PFLEX.isNumber(time) && PFLEX.isString(timeUnit)) {
            if (timeUnit == "month") {
                time = time * 30 * 24 * 60 * 60 * 1000;
            } else if (timeUnit == "year") {
                time = time * 365 * 24 * 60 * 60 * 1000;
            }
            date1 = new Date(date1.getTime() + time);
        }
        return date1;
    },
    createYearOptions: function(elm, min, minTimeUnit, max, maxTimeUnit) {
        var now = new Date(), minDate, maxDate, y;
        minDate = PFLEX.DateUtil.addToDate(now, min, minTimeUnit);
        maxDate = PFLEX.DateUtil.addToDate(now, max, maxTimeUnit);
        for(y = maxDate.getFullYear(); y >= minDate.getFullYear(); y--) {
            elm.options[elm.options.length] = new Option(y, y);
        }
    }
};
// *** phoneutil.js Validator ***

PFLEX.PhoneData = PFLEX.create();

PFLEX.PhoneData.prototype = {
    init: function (isd, phonenumber) {
        this.isd = isd;
        this.phonenumber = phonenumber;
    },
	
};

PFLEX.PhoneUtil = {
    parsePhone : function(value) {
	//value from storage
	if(typeof value === 'undefined' || value === null || value === '' || value === 'undefined') {		
		value = this.getIsdCode()+' ';		
	}
	if(value.indexOf('+') < 0) {
		var isdCode = this.getIsdCode();
		if(value.indexOf(isdCode.substring(1)) === 0) {
			value = value.replace(isdCode.substring(1), '');
		}
		value = isdCode+' '+value;
				
	} else if(value.indexOf('+') === 0) {
		var isdCode = this.getIsdCode();
		if(value.indexOf(isdCode) === 0) {
			value = value.replace(isdCode, '');
		}
		value = isdCode+' '+value;
	}

	var isdCode = value.substring(0, value.indexOf(' '));
	var phoneNumber  = value.substring(value.indexOf(' '));
	phoneNumber = phoneNumber.replace(/\D/g,'');
        return new PFLEX.PhoneData(isdCode, phoneNumber);
    },
    
	format: function(phoneData) {
		if(phoneData == null || typeof phoneData === undefined) {
			return null;
		}
		var phoneNumber = phoneData.phonenumber;
		phoneNumber = phoneNumber.replace(/\D/g,'');
		return phoneData.isd+" "+phoneNumber;
	},
	getIsdCode: function() {
		var countryFieldName = PFLEX.AppInfo.getCountryFieldName();
		var country = PFLEX.StorageUtil.getRegistrationData(countryFieldName);
		if(typeof country === 'undefined' || country === null || country === '' || country === 'undefined') {
			//get country from internal file storage
			country = 'US'
		}
		var isdCode = PFLEX.Options.IsdCodesList[''+country];
		return isdCode;
	},
};
PFLEX.Validator = PFLEX.create();

PFLEX.Validator.prototype = {
    init: function(name, fn){
        if (typeof fn === "function") {
            this._test = fn;
        }
        else {
            this._test = function(){
                return true;
            };
        }
        this.name = name;
    },
    test: function(value, elm, values){
        return this._test(value, elm, values);
    }
};

PFLEX.Validation = PFLEX.create();

PFLEX.Validation.prototype = {
    init: function(options){
        this.options = PFLEX.extend({
            focusOnError: true,
            useCustomMessage: true,
            validateVisibleOnly: false,
            locale: PFLEX.locale
        }, options || {});
    },
    validate: function(elements, values){

        //var errorDiv = document.getElementById('errors');
	//errorDiv.innerHTML = "";
	
	var resultErrorMsg = "";

        if (!PFLEX.isArray(elements) || elements.length == 0) {
            return true;
        }
        var result = true, errors = {}, message = '', focusOnError = this.options.focusOnError, 
		validateVisibleOnly = this.options.validateVisibleOnly, focusElm = null, options = this.options;
        
        elements.pflexEach(function(elm){
            var err = [];
            
            if (!validateVisibleOnly || PFLEX.Validation.isVisible(elm.elm)) {
				
				//var signupField = PFLEX.SignUp.getField(elm.fieldName);
				//signupField.clearErrorMessage();
                
				if (!errors[elm.fieldName]) {
					//console.log("Validation field values == " + elm.fieldName + ":" + values[elm.fieldName]);
					err = PFLEX.Validation.test(elm.elm, options, values, values[elm.fieldName]);
					
					if (err && err.length > 0) {
						if (focusElm === null) {
							focusElm = elm.elm;
						}
						result = false;
						
						//signupField.setErrorMessage(err[0]);
						errors[elm.fieldName] = err;
					}
				}
            }
        });
        
        if (!result) {
            if (!PFLEX.isEmpty(errors)) {
                for (var key in errors) {
	    	    	//errorDiv.innerHTML += errors[key] + "<br>";
		    		resultErrorMsg += errors[key] + "\n";	
                }
            }
            else {
                message = "Validation failed.";
                //errorDiv.appendChild(message);
				resultErrorMsg = "Validation failed."
            }
            //if (focusOnError) {
            //    scroll(0,0);
            //}
        }
        
        return resultErrorMsg;
    }
};

PFLEX.extend(PFLEX.Validation, {
    test: function(elm, options, values, elmValue){
        var valid = true, errors = [], customMessage = options.useCustomMessage, names = new PFLEX.ClassNames(elm);
        
        valid = names.pflexAll(function(value) {
        
            var validator = PFLEX.Validation.get(value), test = true, msg = null, pflexMsg = null;
            
            test = validator.test(elmValue ? PFLEX.trim(elmValue) : elmValue, elm, values);
            if (!test) {
                msg = PFLEX.ValidationErrors.get(validator.name);
                pflexMsg = elm.getAttribute('pflexMessage');
                
                if (pflexMsg) {
                    errors[errors.length] = PFLEX.format(msg, pflexMsg);
                }
                else {
                    errors[errors.length] = msg;
                }
            }
            
            return test;
        });
        
        if (valid) {
            PFLEX.removeClassName(elm, 'validation-failed');
            PFLEX.addClassName(elm, 'validation-passed');
        }
        else {
            PFLEX.removeClassName(elm, 'validation-passed');
            PFLEX.addClassName(elm, 'validation-failed');
        }
        
        return errors;
    },
    isVisible: function(elm){
        if (PFLEX.isString(elm)) {
            elm = PFLEX.$(elm);
        }
        while (elm.nodeName.toLowerCase() != 'body') {
            if (!PFLEX.visible(elm)) {
                return false;
            }
            elm = elm.parentNode;
        }
        return true;
    },
    add: function(validators){
        var nv = {};
        PFLEX.$A(validators).pflexEach(function(value){
            nv[value[0]] = new PFLEX.Validator(value[0], value[1]);
        });
        PFLEX.extend(PFLEX.Validation.methods, nv);
    },
    get: function(name){
        return PFLEX.Validation.methods[name] ? PFLEX.Validation.methods[name] : PFLEX.Validation.methods['_PASS_'];
    },
    methods: {
        '_PASS_': new PFLEX.Validator('_PASS_', '', function(){
            return true;
        })
    },
    tokenToClass: {
	required : "required",
	email : "email",
	usphone : "phone",
	postalcode : "postalcode",
	country : "country",
	phone : "phone",
	requireddateselect : "requireddateselect",
	dateselect : "dateselect",
	childdob : "childdob",
	over18years : "over18years"

    }
});

PFLEX.Validation.add([['empty', function(v){
    return ((v == null) || (v.length == 0));
}
], 
['required', function(v){
    return !PFLEX.Validation.get('empty').test(v);
}], 
['email', function(v){
    return PFLEX.Validation.get('empty').test(v) || /^\w+[\w|\.|\-]*@\w+[\w|\.|\-]*(\.\w{2,})+$/.test(v);
}], 
['postalcode', function(v, elm, values){	
	var countryFieldName = PFLEX.AppInfo.getCountryFieldName();
	var country = 'US';
	if(typeof values === 'undefined' || values === null) {} 
	else {	country = values[countryFieldName];}
	if(typeof country === 'undefined' || country === null || country === '' || country === 'undefined') {
		//get country from internal file storage
		country = PFLEX.StorageUtil.getRegistrationData(countryFieldName);
	}
	if (country == 'US') {
		return PFLEX.Validation.get('empty').test(v) || /(^\d{5}$)|(^\d{5}-\d{4}$)/.test(v);
	} 
	else if (country == 'GB') {
		return PFLEX.Validation.get('empty').test(v) || /^([Gg][Ii][Rr] 0[Aa]{2})|((([A-Za-z][0-9]{1,2})|(([A-Za-z][A-Ha-hJ-Yj-y][0-9]{1,2})|(([A-Za-z][0-9][A-Za-z])|([A-Za-z][A-Ha-hJ-Yj-y][0-9]?[A-Za-z])))){0,1}[0-9][A-Za-z]{2})$/.test(v);
		//as found at http://regexlib.com/REDetails.aspx?regexp_id=260
	}
	else if (country == 'IN') {
		return PFLEX.Validation.get('empty').test(v) || /(^\d{6}$)/.test(v);
	}
	else if (country == 'CA') {
		return PFLEX.Validation.get('empty').test(v) || /(^[ABCEGHJKLMNPRSTVXY]\d[A-z]\s?\d[A-z]\d$)/.test(v);
	}
	else if (country == 'KR') {
		return PFLEX.Validation.get('empty').test(v) || /(^\d{3})(-|\s)?(\d{3}$)/.test(v);
	}
	else {
		return true;
	}
}], 
['phone', function(v, elm, values){
	var countryFieldName = PFLEX.AppInfo.getCountryFieldName();
	var country = 'US';
	if(typeof values === 'undefined' || values === null) {} 
	else {	country = values[countryFieldName];}
	if(typeof country === 'undefined' || country === null || country === '' || country === 'undefined') {
		//get country from internal file storage
		country = PFLEX.StorageUtil.getRegistrationData(countryFieldName);
	}

	if (country == 'US') {
		return PFLEX.Validation.get('empty').test(v) || /^1*.*\d\d\d.*\d\d\d.*\d\d\d\d$/.test(v);
	} else {
		return PFLEX.Validation.get('empty').test(v) || /^(([0-9]{1})*[- .(]*([0-9a-zA-Z]{3})*[- .)]*[0-9a-zA-Z]{3}[- .]*[0-9a-zA-Z]{4})/.test(v);
	}
}], 
['usphone', function(v, elm){
	return PFLEX.Validation.get('empty').test(v) || /^1*.*\d\d\d.*\d\d\d.*\d\d\d\d$/.test(v);
}], 
['country', function(v, elm){
    var countries = PFLEX.Options.country;
	for(var key in countries) {
		if(countries[key] == v) {
			return true;
		}
	}
	return false;
}],
['requireddateselect', function(v, elm){
	var options = elm.getElementsByTagName('select');
    return PFLEX.$A(options).pflexAll(function (elm) {
        return elm.options ? elm.selectedIndex > 0 : false;
    });
}],
['dateselect', function(v, elm){
	var parsedDate = PFLEX.DateUtil.parseDateFromForm(elm), selectedDate;
    selectedDate = new Date(parsedDate.year, parsedDate.month - 1, parsedDate.day);
    return selectedDate.getFullYear() == parsedDate.year &&
        	selectedDate.getMonth() == (parsedDate.month - 1) &&
        	selectedDate.getDate() == parsedDate.day;
}],
['childdob', function(v, elm) {
    var parsedDate = PFLEX.DateUtil.parseDateFromForm(elm), now = new Date();
    var dateDiff = PFLEX.DateUtil.dateDiff(new PFLEX.DateData(now.getFullYear(), now.getMonth() + 1, now.getDate()), parsedDate);
    if (dateDiff.year > 0) {
        return (dateDiff.year < 2 || (dateDiff.year == 2 && dateDiff.month == 0));
    } else {
        return (dateDiff.year == 0 && dateDiff.month >= -9);
    }
}],
['over18years', function(v, elm) {
    var parsedDate = PFLEX.DateUtil.parseDateFromForm(elm), now = new Date();
    var dateDiff = PFLEX.DateUtil.dateDiff(new PFLEX.DateData(now.getFullYear(), now.getMonth() + 1, now.getDate()), parsedDate);
    return dateDiff.year >= 18;
}]
]);

PFLEX.ValidationErrors = PFLEX.create();

PFLEX.extend(PFLEX.ValidationErrors, {
    add: function(locale, messages){
        var lm = {};
        PFLEX.$A(messages).pflexEach(function(value){
            lm[locale + "-" + value[0]] = value[1];
        });
        PFLEX.extend(PFLEX.ValidationErrors.messages, lm);
    },
    get: function(message){
        var name;
        if (message) {
            name = PFLEX.locale + "-" + message;
            if (PFLEX.ValidationErrors.messages[name]) {
                return PFLEX.ValidationErrors.messages[name];
            }
            return PFLEX.ValidationErrors.messages[PFLEX.defaultLocale + '-' + message];
        }
        return PFLEX.ValidationErrors.messages['_DEFAULT_'];
    },
    messages: {
        '_DEFAULT_': 'This value is required.'
    }
});

PFLEX.ValidationErrors.add('en', 
	[['required', '{0} is required.'], 
	['email', '{0} is not valid. Please enter a valid email address.'], 
	['postalcode', 'Please enter a valid postal code.'],
	['phone', '{0} must be a valid phone number.'],
	['country', '{0} must be a valid country.'],
	['requireddateselect', '{0} is required.'],
	['dateselect', 'Please select a valid {0}.'],	
	['over18years', 'You must be 18 years or older.'],
	['childdob', 'Please enter a child birth date of under 2 years or valid due date.']]);

PFLEX.ValidationErrors.add('es', 
	[['required', '{0} es necesario.'], 
	['email', '{0} no es válida. Por favor, introduzca una dirección de email válidao.'], 
	['postalcode', '{0} no es válido.'], 
	['phone', '{0} no válido.'],
	['country', '{0} no es válido.'],
	['requireddateselect', '{0} es necesario.'],
	['dateselect', '{0} no es válido.'],		
	['over18years', 'Tienes que tener 18 años o más.'],
	['childdob', 'Por favor debes dar la fecha de nacimiento de un niño menor de 2 años o una fecha válida.']]);
// *** Placement ***

PFLEX.Placement = PFLEX.create();

PFLEX.Placement.prototype = {
	init: function (pid, options) {
		this.id = pid;
		this.width = 0;
		this.height = 0;
		this.maxOffers = Number.MAX_VALUE;
		this.minOffers = 1;
        this.offerRegistry = {};
        this.adType = null;

        this.options = PFLEX.extend({
            data: {},
            trackingURL: null,
            redirectURL: null,
            redirectTarget: null,
            subSourceId: null,
            numOffers: Number.MAX_VALUE,
            locale: PFLEX.locale
        }, options || {});

        this.formData = this.options.data;
        this.trackingURL = this.options.trackingURL;
        this.redirectURL = this.options.redirectURL;
        this.redirectTarget = this.options.redirectTarget;
        this.subSourceId = this.options.subSourceId;
        this.numOffers = this.options.numOffers;
        this.locale = this.options.locale;
        
		this.totalWeight = 0;
		this.DOMAnchorId = null;
		this.offerDOMAnchorId = null;
	
		this.submitURL = "@appweb_base_url@/bpost";
	
		this._renderTimeout = null;
		this._firstOfferSelected = false;

        this.semaphore = {
            counter: 0,
            max: 0,
            init: function(count){ this.max = count; this.counter = 0; },
            locked: function() { return this.counter < this.max; },
            signal: function() { this.counter++; }
        };
    },
    isAdUnitx: function () {
        if (this.adType == "AdUnitx") {
            return true;
        }
        return false;
    },
    isCoreg: function() {
        if (this.adType == "CoReg") {
            return true;
        }
        return false;
    },
    getDOMAnchor: function () {
        if (!this.DOMAnchorId) { return null; }
        return PFLEX.$(this.DOMAnchorId);
    },
    setDOMAnchor: function (id) {
        if (typeof id != "string") { return; }
        this.DOMAnchorId = id;
    },
    getOfferDOMAnchor: function () {
        if (!this.offerDOMAnchorId) { return this.getDOMAnchor(); }
        return PFLEX.$(this.offerDOMAnchorId);
    },
    setOfferDOMAnchor: function (id) {
        if (typeof id != "string") { return; }
        this.offerDOMAnchorId = id;
    },
    isFirstOfferSelected: function () {
        return this._firstOfferSelected;
    },
    setFirstOfferSelected: function (bool) {
        if (bool === true || bool === false) {
            this._firstOfferSelected = bool;
        }
    },
    writeAnchor: function () {
        var anchor;
        if (this.options.anchorId) {
            anchor = d.getElementById(this.options.anchorId);
            if (anchor) {
                anchor.innerHTML = "";
                anchor.id = this.DOMAnchorId;
            }
        }
        if (!anchor) { d.write("<div id='" + this.DOMAnchorId + "' style='position:relative;'></div>"); }
        this.getDOMAnchor().style.width = this.width + "px";
        this.getDOMAnchor().style.height = this.height + "px";
    }, 
    registerOffer: function (oid) {
        if (PFLEX.isUndefined(this.offerRegistry[oid])) {
            this.offerRegistry[oid] = new PFLEX.Offer(this.id, oid, {
                path: this.path
            });
        }
    }, 
    registerOffer: function (oid, offer) {
        if (PFLEX.isUndefined(this.offerRegistry[oid])) {
            this.offerRegistry[oid] = offer;
        }
    },
    hasSelectedOffer: function () {
        for (var oid in this.offerRegistry) {
            // Added to avoid pflex functions added to the native Object prototype
            if (this.offerRegistry.hasOwnProperty(oid)) {
                var o = this.offerRegistry[oid];
                if (o.loaded && !o.submitted && o.isSelected()) {
                    return true;
                }
            }
        }
        return false;
    },
	getOffers : function () {
		var offers = [];
		for (var oid in this.offerRegistry) {
            		// Added to avoid pflex functions added to the native Object prototype
	            	if (this.offerRegistry.hasOwnProperty(oid)) {
        	    	    var o = this.offerRegistry[oid];
				offers.push(o);
            	    	}
        	}
		return offers;
	},
    getSelectedOffers : function() {
	for (var oid in this.offerRegistry) {
            // Added to avoid pflex functions added to the native Object prototype
            if (this.offerRegistry.hasOwnProperty(oid)) {
                var o = this.offerRegistry[oid];
                if (o.loaded && !o.submitted && o.isSelected()) {
                    return true;
                }
            }
        }
        return false;
    },
    toggleOfferState: function(oid) {
	var offer = this.offerRegistry[oid];
	if(offer) {
		var chkbox = document.getElementById("chkbox_"+oid);
		if(offer.isSelected() === 'true') {
			offer.setSelected('false');
			chkbox.checked = false;
		} else {
			offer.setSelected('true');
			chkbox.checked = true;
		}
	}
    },
    offerCheckIn: function (oid, divId) {
        if (!PFLEX.isUndefined(this.offerRegistry[oid])) {
            this.offerRegistry[oid].setDOMObject(divId);
            this.offerRegistry[oid].loaded = true;
        }
    },
	trackImpression: function () {
		var url;
		if (PFLEX.stats) {
			url = PFLEX.stats + (this.isCoreg() ? "/reg/placement" : "/imp/placement");
			url += ("/" + this.id);
			if (PFLEX.isString(this.subSourceId)) url += ("/" + this.subSourceId);
			PFLEX.renderPixel(url, true);
		}
	},
};
// *** Common offer.js***

PFLEX.Offer = PFLEX.create();

PFLEX.Offer.prototype = {
	init: function (offerjson, selected, posted, showAdditionalFieldsForm) {
		this.offerjson = offerjson;
		this.selected = selected;
		this._DOMObjectId = null;
		this.signUpFields = {};
		this.initializeFields();
		this.posted = posted;
        this.showAdditionalFieldsForm = showAdditionalFieldsForm;
	},
	initializeFields: function() {
		var fieldNameJson = this.offerjson.registrationFields;
        	if(typeof fieldNameJson !== "undefined" && fieldNameJson !== null) { 
			for( var key in fieldNameJson ) {
				var fieldJson = JSON.parse(JSON.stringify(fieldNameJson[key]));
				var validations = [];
				var validationsJson = JSON.parse(JSON.stringify(fieldJson.validations));
				for(var i=0;i<validationsJson.length;i++) {
					validations.push(PFLEX.Validation.get(validationsJson[i]));
					if (validationsJson[i].toLowerCase() == "postalcode") {
                        	this.postalCodeFieldName = key;
                    }
					if (validationsJson[i].toLowerCase() == "country") {
                            this.countryFieldName = key;
                    }
				}				
				var field = new PFLEX.SignUpField(key, fieldJson.displayName, fieldJson.order, '', fieldJson.type, validations, '', '', "false", 
												fieldJson.optionType, fieldJson.transientField, fieldJson.options, false, fieldJson.fieldType);
        		this.signUpFields[key] = field;
			}
			if(this.postalCodeFieldName !== null && typeof this.postalCodeFieldName !== 'undefined') {
				if(this.countryFieldName == null || typeof this.countryFieldName == 'undefined') {
					var countryField = {};
					countryField["displayName"] = "Country";
					countryField["order"] = fieldNameJson[this.postalCodeFieldName].order;
					countryField["type"] = "select";
					countryField["optionType"] = "country";
					countryField["validations"] = ["required", "country"];
					this.countryFieldName = "country";
					fieldNameJson[this.countryFieldName] = countryField;
					this.offerjson.registrationFields = fieldNameJson;
				}
			}
		}
	},
	getText1 : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.offerText1;
	},	
	getText2 : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.offerText2;
	},
	getHeaderText : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.headerText;
	},
	getBodyText : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.bodyText;
	},
	getPrivacyUrl : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.privacyPolicyUrl;
	},
	getImagePath : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.logoPath;
	},
	getOfferJson : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return JSON.stringify(this.offerjson);
		return "";	
	},
	getOid : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.oid;
		return "-1";
	},
	getCreativeId : function() {		
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.creativeId;
	},
	getDealId : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.dealId;
	},
	getConfirmHeader : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.confirmHeader;
	},
	getConfirmBody : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.confirmBody;
	},
	isSelected: function () {
        return this.selected;
    },
    setSelected: function (selected) {
        this.selected = selected;
    },
	getRegistrationFields: function() {
		if(typeof this.offerjson == "undefined" && this.offerjson == null)
			return null;
		return this.offerjson.registrationFields;
	},
	hasValidSignUpFields: function() {
		if(this.signUpFields === null || typeof this.signUpFields === 'undefined') {
			return true;
		}
		for(var key in this.signUpFields) {
			var signUpField = this.signUpFields[key];
			var validations = signUpField.validations;
			if(validations == null || validations.length == 0) {
				continue;
			}
			var isRequired = false;
			for(var i=0; i<validations.length; i++) {
				var validation = validations[i].name;
				if(''+validation === 'required') {
					isRequired = true;
					break;
				}
			}
			if(isRequired) {
				if(signUpField.value == null || typeof signUpField.value == 'undefined' || signUpField.value === '' || signUpField.value === 'undefined') {
					return false;
				}
			}
		}
		return true;
	},
	getFieldsForRendering: function() {
		if(this.signUpFields == null || typeof this.signUpFields == "undefined" || this.signUpFields.length == 0) {
			return null;
		}
		var regFields = this.getRegistrationFields();
		var placementRegFields = PFLEX.AppInfo.getRegistrationFields();
		if(regFields == null || typeof regFields == 'undefined')
			return null;
		var fields = {};
		for(var key in regFields) {
			var field = this.signUpFields[key];
			//fix for undefined values
			if(field) {
				if(field.value === "undefined") {
					field.value = "";
				}
			}
			var fieldAdded = false;

			
			var placementField = placementRegFields[key];
			if(placementField) {
				
			} else {
				fieldAdded = true;
				fields[key] = regFields[key];

			}
			
			if(key == this.postalCodeFieldName && fieldAdded == true) {
				fields[this.countryFieldName] = regFields[this.countryFieldName];
			}
			if(fieldAdded) {
				if(field) { field.markedForOfferRendering = true; }
			}
		}
		if(this.postalCodeFieldName != null && typeof this.postalCodeFieldName != "undefined") {
			fields =  this.clusterFields(fields, [this.getPostalCodeFieldName(), this.getCountryFieldName()]);
		}
		var fieldsList = [];
		for( var key in fields ) {
			if(typeof key !== 'undefined' && key !== null && key !== '') {
				var field = PFLEX.extend(fields[key], { fieldName : key } );
				//console.log('=====Inside generating fieldList for offer oid='+this.getOid()+'=======key='+key+'=========');
				fieldsList.push(field);
			}
		}
		return this.getSortedFields(fieldsList);
	},
	getOfferAdditionalFields: function() {
		return this.getFieldsForRendering();

	},
	clusterFields : function(fieldsObj, fieldNames) {
		var cluster_field = {};
		var cluster_field_name = '';
		cluster_field["type"] = "composite";
		cluster_field["fields"] = {};
		for (var i = 0; i < fieldNames.length; i++) {
			if (fieldsObj.hasOwnProperty(fieldNames[i])) {
				var field = fieldsObj[fieldNames[i]];
				if(i==0) {
					cluster_field["order"] = field.order;
				}
				cluster_field.fields[fieldNames[i]] = fieldsObj[fieldNames[i]];
				field.order = i+1;
				delete fieldsObj[fieldNames[i]];
				cluster_field_name = cluster_field_name + '_' + fieldNames[i];
			}
		}
		fieldsObj[cluster_field_name] = cluster_field;
		return fieldsObj;
	},
	getSortedFields: function (fields) {
		return fields.sort(function(a, b){
			var compValue = a.order - b.order;
			if(compValue == 0) {
				if(a.type == "composite") {
					if(b.type == "composite") {
						if(Math.floor(Math.random()*2) == 0) return -1;
						else return 1;
					} else {
						return 1;
					}
				} else {
					if(b.type == "composite") {
						return -1;
					}
				}
				if(PFLEX.isString(a.displayName) & PFLEX.isString(b.displayName)) {
					if(a.displayName == b.displayName){
						return 0;
					}
					return (a.displayName < b.displayName) ? -1 : 1;
				}
			}
			return compValue;
		});
	},
	getPostalCodeFieldName: function() {
		return this.postalCodeFieldName;;
	},
	getCountryFieldName: function() {
		return this.countryFieldName;
	},
};
// *** Container.js ***

PFLEX.Ads = PFLEX.create();

PFLEX.extend(PFLEX.Ads, {
    addEvent: function (id, options) {
        var ne = { };
        ne[id] = new PFLEX.EventHandler(id, options);
        PFLEX.extend(PFLEX.Ads.events, ne);
    },
    addPlacement: function (id, options) {
        var np = { };
        np[id] = new PFLEX.Placement(id, options);
        PFLEX.extend(PFLEX.Ads.placements, np);
    },
    placement: function(id) {
        return PFLEX.Ads.placements[id] ? PFLEX.Ads.placements[id] : { };
    },
    offer: function(id, oid) {
        return PFLEX.Ads.placement(id).offerRegistry[oid];
    },
    placements: { },
});
// *** Common privacyurl.js***

PFLEX.PrivacyUrl = PFLEX.create();

PFLEX.PrivacyUrl.prototype = {
	init: function(url, selected){
		this.url = url;
		this.selected = selected;
	},
	getUrl: function(){
		return this.url;
	},
	isSelected: function(){
		return selected;
	},
	setSelected: function(selected){
		this.selected = selected;
	}
}

PFLEX.PrivacyPageConfig = PFLEX.create();

PFLEX.PrivacyPageConfig.prototype = {
	init: function(urlToRender, signUpCase, signUpPageType, privacyUrls, privacyPageType, pageTitle){
		this.urlToRender = urlToRender;		
		this.signUpCase = signUpCase;
		this.signUpPageType = signUpPageType;
		this.privacyUrls = privacyUrls;
		this.privacyPageType = privacyPageType;
		this.pageTitle = pageTitle;
	},
	isSignUpCase: function(){
		return this.signUpCase; //true or false
	},
	getSignUpPageType: function(){
		return this.signUpPageType;
	},
	getPrivacyUrls: function(){
		return this.privacyUrls;
	},
	getUrlToRender: function() {
		return this.urlToRender;
	},
	setUrlToRender: function(urlToRender) {
		this.urlToRender = urlToRender;
	},
	getPrivacyPageType: function() {
		return this.privacyPageType; //privacy or privacylist
	},
	setPrivacyUrls: function(privacyUrls){
		this.privacyUrls = privacyUrls;
	},
	setSignUpPageType: function(signUpPageType){
		this.signUpPageType = signUpPageType;
	},
	setPrivacyPageType: function(privacyPageType) {
		this.privacyPageType = privacyPageType;
	},
	getPageTitle: function() {
		return this.pageTitle;
	},
	setPageTitle: function(pageTitle) {
		this.pageTitle = pageTitle;
	},


}

PFLEX.PrivacyActions = PFLEX.create();

PFLEX.extend(PFLEX.PrivacyActions, {

	getPontiflexPrivacyPolicyUrl: function() {
		return "http://www.pflexads.com/privacy_policy.html";
	},

	showPrivacyPage : function(signUpCase, multiofferSignupPage, offerSignupPage) {

		var privacyPageUrl = PFLEX.PrivacyActions.getPontiflexPrivacyPolicyUrl();
		var privacyPageConfig = new PFLEX.PrivacyPageConfig(privacyPageUrl, signUpCase, 'registration', '', 'privacy', PFLEX.StringHelper.get('privacy_policy'));
		if(signUpCase || multiofferSignupPage) {
			PFLEX.StorageUtil.saveRegFormState();
			var signupPageType = PFLEX.SignUp.getPageType();
			if(typeof signupPageType !== "undefined" && signupPageType === "missinginfo") {
				PFLEX.PageTransitionState.setSignupPageType('missinginfo');
				privacyPageConfig.setSignUpPageType(signupPageType);
			}
			
			//check if its registration page, if yes use privacy policy url from registration offer
			var signupPageTypeInconfig = privacyPageConfig.getSignUpPageType();
			if(typeof signupPageTypeInconfig !== "undefined" && signupPageTypeInconfig === "registration") {
				var regOffer = PFLEX.AppInfo.getOffer();
				if(typeof regOffer !== "undefined" && regOffer !== null) {
					var regOfferPrivacyUrl = regOffer.getPrivacyUrl();
					if(typeof regOfferPrivacyUrl !== "undefined" && regOfferPrivacyUrl !== null) {
						privacyPageConfig.setUrlToRender(''+regOfferPrivacyUrl);
					}
				}
			}

		}
		if(offerSignupPage) {
			privacyPageConfig.setSignUpPageType('offer_signup');
		}
		if(!signUpCase) {
			//multioffer page, store urrent offers state
			PFLEX.StorageUtil.saveOffersState();

			//set privacy urls
			var urls = [];
			var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
			var offersList = placement.getOffers();
			for(var key in offersList) {
				var o = offersList[key];
				var selected = o.selected;
				if(typeof selected !== "undefined" && selected !== null) {
					var privacyurl = new PFLEX.PrivacyUrl(o.getPrivacyUrl(), false);
					urls.push(privacyurl);		
				}
			}
			privacyPageConfig.setPrivacyUrls(urls);

		}
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		//load privacy page
		location.replace('./privacy.html');

	},
	performPrivacyCloseButton: function() {
		var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();	
		var signUpCase = privacyPageConfig.isSignUpCase(); 
		var pageType = privacyPageConfig.getSignUpPageType();
		PFLEX.PageTransitionState.clearPrivacyPageConfig();
		
		if(pageType === 'offer_signup') {
			location.replace('./offer_signup.html');
		} else if(signUpCase) {
			//load signup page
			location.replace('./registration.html');
		} else {
			//load multioffer page
			location.replace('./multioffer.html');
		}
	},

	showPrivacyListPage : function() {
		//fetch privacyPageConfig
		var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();
		privacyPageConfig.setPrivacyPageType('privacylist');
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		location.replace('./privacy.html');
	},
	peformPrivacySelectButton : function() {
		//fetch privacyPageConfig
		var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();
		privacyPageConfig.setPrivacyPageType('privacy');
		
		//set url-to-render
		var urlToRender = PFLEX.PrivacyActions.getPontiflexPrivacyPolicyUrl();
		var privacyurls = privacyPageConfig.getPrivacyUrls();
		for(var key in privacyurls) {
			var targeturl = privacyurls[key].url;
			if(typeof targeturl !== "undefined" && targeturl !== null) {
				if(privacyurls[key].selected) {
					urlToRender = targeturl;		
				}
			}			
		}
		privacyPageConfig.setUrlToRender(urlToRender);
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		location.replace('./privacy.html');
	},
	getPrivacyPageConfig: function() {
		return this.config[0];
	},
	setPrivacyPageConfig: function(privacyPageConfig) {
		this.config[0] = privacyPageConfig;
	},
	config: {},
	performUrlSelection: function(url) {
		var privacyPageConfig = this.config[0];
		var privacyurls = privacyPageConfig.getPrivacyUrls();
		for(var key in privacyurls) {
			var targeturl = privacyurls[key].url;
			if(typeof targeturl !== "undefined" && targeturl !== null) {
				privacyurls[key].selected = false;
				if(targeturl === url) {
					privacyurls[key].selected = true;
				}
			}			
		}
	}
	
});
// *** Common versionhelper.js***

PFLEX.VersionHelper = {

getPflxAdVersion: function() {
	return "3";
},
getPflxSdkVersionCode: function() {
	return encodeURI("4.0.0");
},
getPflxSdkVersionName: function() {
	return "AndroidSDK";
},
getPflxAdSdkVersion: function() {
	var result = PFLEX.NativeVersion.getOSType();
	result = result + " " + this.getOsVersion();
	result = result + " Sdk " + this.getPflxSdkVersionName();

	return result;
},
getAppName: function() {
	return PFLEX.NativeVersionHelper.getAppName();
},
getAppVersionCode: function() {
	return PFLEX.NativeVersionHelper.getAppVersionCode();
},
getAppVersionName: function() {
	return PFLEX.NativeVersionHelper.getAppVersionName();
},
getAppUri: function() {
	var result = "app://"+ encodeURI(this.getAppName());
	result = result +"/"+this.getAppVersionCode();
	result = result +"/"+this.getAppVersionName();

	return result;
},
getOsVersion: function() {
	return PFLEX.NativeVersionHelper.getOsVersion();
},
getDNT: function() {
	return "0";
},
getUserAgent: function() {
	return PFLEX.NativeVersionHelper.getUserAgent();
},
getLocale: function() {
	return PFLEX.NativeVersionHelper.getLocale();
},
getPostalCode: function() {
	return PFLEX.NativeVersionHelper.getPostalCode();
},
getHeaders: function() {
	headers = {};
	headers["X-Pflex-Ad-Version"] = ''+this.getPflxAdVersion();
	headers["X-Pflex-Ad-Sdk-Version"] = ''+this.getPflxSdkVersionCode();
	headers["X-Pflex-Referer"] = ''+this.getAppUri();
	headers["X-Pflex-Agent"] = ''+this.getUserAgent();
	headers["Accept-Language"] = ''+this.getLocale();

	var postalCode = this.getPostalCode();
	if(typeof postalCode !== "undefined" && postalCode !== null) {
		headers["X-Pflex-Postalcode"] = ''+postalCode;
	}	
	return headers;
}

};
// *** Common updateResource.js***
PFLEX.UpdateResource = PFLEX.UpdateResource || {};

PFLEX.UpdateResource.Resource = function (config, updateResManager) {
    var newResource = {

        config: config,

        updateResourceMgr: updateResManager,

        updateResource: function (updateJsonObj) {
            //var resourceKey = this.config.key;
            //update lastCheckedDate
            this.updateLastCheckedDate();

            var proceedWithUpdate = false;
            //method call for checkForForceUpdate
            if (this.checkForForceUpdate(updateJsonObj)) {
                proceedWithUpdate = true;
            } else {
                //method call to check for version
                if (this.isNewVersionAvailable(updateJsonObj)) {
                    proceedWithUpdate = true;
                }
            }
            if (proceedWithUpdate) {
                //check if isUpdateInProgress = true, if yes exit
                if ('' + this.isUpdateInProgress() === "true") {
                    return;
                }
                //register events
                this.registerResourceUpdateEvent(updateJsonObj);
                //set updateInProgress = true
                this.setUpdateInProgress(true);
                //make native call to update resource
                this.updateResourceInNative(updateJsonObj);
            }
            //after receiving onUpdateOfResource event, do following
            //set new versionCode, lastUpdatedDate, updateInProgress = false
            //clear data from old location
        },

        updateLastCheckedDate: function () {
            //make native call passing ResourceData location file
            var currentDate = PFLEX.DateUtil.getUTCNow();
            var lastCheckedDateStr = PFLEX.DateUtil.formatAsTimestamp(currentDate);
            var reslastCheckedDateKey = this.config.key + PFLEX.UpdateResource.config.lastCheckedDateSuffix;
            PFLEX.NativeResource.setResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, reslastCheckedDateKey, lastCheckedDateStr);

        },
        getResourceUpdateJsonObj: function (updateJsonObj) {
            return updateJsonObj[this.config.key];
        },

        checkForForceUpdate: function (updateJsonObj) {
            //check in  updateJsonObj 
            var resourceUpdateObj = this.getResourceUpdateJsonObj(updateJsonObj);
            var isForceUpdate = false;
            if (typeof resourceUpdateObj !== "undefined" && resourceUpdateObj !== null) {
                isForceUpdate = resourceUpdateObj["forceupdate"];
            }
            return isForceUpdate;
        },

        isNewVersionAvailable: function (updateJsonObj) {
            //check if new version present 
            var resourceUpdateObj = this.getResourceUpdateJsonObj(updateJsonObj);
            var newVersionCode = null;
            if (typeof resourceUpdateObj !== "undefined" && resourceUpdateObj !== null) {
                newVersionCode = resourceUpdateObj["versioncode"];
            }
            var resVersionCodeKey = this.config.key + PFLEX.UpdateResource.config.versionCodeSuffix;
            var resVersionCode = PFLEX.NativeResource.getResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, resVersionCodeKey);

            //there is no resource version code saved, so it is first call for update
            if (typeof resVersionCode === "undefined" || resVersionCode === null || "" === resVersionCode) return true;

            return this.compareVersion(newVersionCode, resVersionCode);
        },
        compareVersion: function (newVersion, currentVersion) {
            var result = false;

            if (typeof newVersion === "undefined" || newVersion === null || "" === newVersion) return result;

            if (typeof currentVersion === "undefined" || currentVersion === null || "" === currentVersion) return result;

            //version can be in format 1.0.4
            var currentVersionIdentifiers = (currentVersion + '').split('.');
            var newVersionIdentifiers = (newVersion + '').split('.');

            //check major, minor and patch version
            for (var key in newVersionIdentifiers) {
                var newInt = Number(newVersionIdentifiers[key]);
                var currentInt = -1;
                //this is case when current version is like 1.0.3 and new version is 1.0.3.1
                if (currentVersionIdentifiers.length > key) {
                    currentInt = Number(currentVersionIdentifiers[key]);
                }
                if (newInt > currentInt) {
                    result = true;
                }
            }
            return result;
        },

        isUpdateInProgress: function () {
            //make native call passing ResourceData location file 
            var resUpdateInProgressKey = this.config.key + PFLEX.UpdateResource.config.isUpdateInProgressSuffix;
            var result = PFLEX.NativeResource.getResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, resUpdateInProgressKey);
            return result;

        },

        setUpdateInProgress: function (value) {
            //make native call passing ResourceData location file 
            var resUpdateInProgressKey = this.config.key + PFLEX.UpdateResource.config.isUpdateInProgressSuffix;
            PFLEX.NativeResource.setResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, resUpdateInProgressKey, '' + value);
        },

        updateResourceInNative: function (updateJsonObj) {
            //make native call passing resourceName, resourceDownloadTempPath, resourcePath, expectedChecksum
            var updateConfig = updateJsonObj[this.config.key];
            var url = PFLEX.AppInfo.getAppInfoPflxAdUrl() + updateConfig.url;
            var resourcePath = PFLEX.UpdateResource.config.resourceBasePath + '/' + this.config.name;
            var resourceTempPath = PFLEX.UpdateResource.config.resourceTempPath + '/' + this.config.name;
            var resourceOldPath = PFLEX.UpdateResource.config.resourceOldPath + '/' + this.config.name;
            var headers = JSON.stringify(PFLEX.VersionHelper.getHeaders());
            PFLEX.NativeResource.updateResource(url, updateConfig.hash, resourcePath, resourceTempPath, resourceOldPath, headers, updateConfig.extract, this.config.key);
        },

        setVersionCode: function (updateJsonObj) {
            //make native call passing ResourceData location file 
            var resourceUpdateObj = updateJsonObj[this.config.key];
            var newVersionCode = null;
            if (typeof resourceUpdateObj !== "undefined" && resourceUpdateObj !== null) {
                newVersionCode = resourceUpdateObj["versioncode"];
            }
            var resVersionCodeKey = this.config.key + PFLEX.UpdateResource.config.versionCodeSuffix;

            PFLEX.NativeResource.setResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, resVersionCodeKey, '' + newVersionCode);
        },

        setLastUpdatedDate: function () {
            //make native call passing ResourceData location file
            var currentDate = PFLEX.DateUtil.getUTCNow();
            var lastUpdatedDateStr = PFLEX.DateUtil.formatAsTimestamp(currentDate);
            var resLastUpdatedDateKey = this.config.key + PFLEX.UpdateResource.config.lastUpdatedDateSuffix;
            PFLEX.NativeResource.setResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, resLastUpdatedDateKey, lastUpdatedDateStr);
        },


        registerResourceUpdateEvent: function (updateJsonObj) {
            PFLEX.UpdateResource.eventHandler = new PFLEX.EventHandler();
	    var mgr = this.updateResourceMgr;
            PFLEX.UpdateResource.eventHandler.register("onUpdateOfResource", function (eventName, stringFromReg, unsure, objFromFire) {
                var resKey = objFromFire.resourceKey;
                var success = objFromFire.updateSuccessful;
                var resource = null;
                PFLEX.$A(mgr.resources).pflexEach(function (resourceTmp) {
                    var resKeyTmp = resourceTmp.config.key;
                    if (typeof resKeyTmp !== "undefined" && resKeyTmp !== null && resKeyTmp === resKey) resource = resourceTmp;
                });
                if (resource !== null) {
			//console.log("Resource update successfull: "+ success+ " for key: " + resource.config.key);
                	if (success == true) {
				console.log("Resource update was successful for key: " + resource.config.key);
                        	resource.setVersionCode(updateJsonObj);
                        	resource.setLastUpdatedDate();
                    	} else {
				console.log("Resource update failed for key: " + resource.config.key);
			}
                    	resource.setUpdateInProgress(false);
                }
            });

        },
    };
    return newResource;

};

PFLEX.UpdateResource.Resource.jsclient = PFLEX.UpdateResource.Resource;

PFLEX.UpdateResource.Resource.appinfojson = function (config, updateResManager) {
    var newResource = PFLEX.UpdateResource.Resource(config, updateResManager);
    newResource.checkForForceUpdate = function (updateJsonObj) {
        return false;
    }, newResource.isNewVersionAvailable = function (updateJsonObj) {
        return true;
    }, newResource.setVersionCode = function (updateJsonObj) {
        return;
    }
    return newResource;
};


// *** Common updateResourceConfig.js***
PFLEX.UpdateResource = PFLEX.UpdateResource || {};

PFLEX.UpdateResource.config = {
    lastCheckedDateSuffix: "_lastCheckedDate",
    lastUpdatedDateSuffix: "_lastUpdatedDate",
    versionCodeSuffix: "_versionCode",
    isUpdateInProgressSuffix: "_isUpdateInProgress",
    updateUrlKey: "/update/resource/android/",

    resourceBasePath: "pflx_res",
    resourceTempPath: "pflx_res_temp",
    resourceOldPath: "pflx_res_old",
    resourceDataFilePath: "PflxResourceData",
    //frequency is modelled as dd hh mm ss. it helps in combination of time units
    resources: [{
        "key": "appinfojson",
        "frequency": "1 00 00 00",
        "name": "AppInfo.json",
        "extract": false,
        "updateResource": PFLEX.UpdateResource.Resource.appinfojson
    }, {
        "key": "jsclient",
        "frequency": "1 00 00 00",
        "name": "js-client.zip",
        "extract": true,
        "updateResource": PFLEX.UpdateResource.Resource.jsclient
    }]

};

// *** Common updateResourceManager.js***
PFLEX.UpdateResource = PFLEX.UpdateResource || {};


PFLEX.UpdateResource.Manager = {
    resources: [],
    //initialization code for populating resources                
    initResourcesFromConfig: function () {
	var mgr = PFLEX.UpdateResource.Manager;
        PFLEX.$A(PFLEX.UpdateResource.config.resources).pflexEach(function (resourceConfig) {
            mgr.resources.push(resourceConfig.updateResource(resourceConfig, mgr));
        });
    },

    //array of PFLEX.UpdateResouce.Resource object array which are configured from config
    resourcesForUpdate: [],
    //array of PFLEX.UpdateResouce.Resource object array which need update				
    updateResources: function () {
        //get all resources which need update using getResourcesRequiringUpdate
        var resourcesForUpdateTmp = this.getResourcesRequiringUpdate();
        var toUpdate = false;
	var mgr = PFLEX.UpdateResource.Manager;
        PFLEX.$A(resourcesForUpdateTmp).pflexEach(function (resource) {
            toUpdate = true;
            mgr.resourcesForUpdate.push(resource);
        });
        //get update json by making async call  checkForUpdates			 
        //it will make call to callback method

	updateResourcesCallBack = function (jsonStr) {
	        //create updateJsonObj from result
	        var updateJsonObj = null;
        	if (typeof jsonStr !== "undefined" && jsonStr !== null) {
        	    updateJsonObj = JSON.parse('' + jsonStr);
        	}
        	if (updateJsonObj === null) return;
        	
        	var resourcesForUpdateTmp = mgr.resourcesForUpdate;
        	PFLEX.$A(resourcesForUpdateTmp).pflexEach(function (resource) {
        	        resource.updateResource(updateJsonObj);
        	});

    	};

        if (toUpdate) {
            this.checkForUpdates(PFLEX.AppInfo.getAppInfoPflxAdUrl(), PFLEX.AppInfo.getPid(), updateResourcesCallBack);
        }
    },
    
    isUpdateRequired: function (currentDate, lastCheckedDate, resourceConfig) {
        var diffInSeconds = PFLEX.DateUtil.differenceInSeconds(currentDate, lastCheckedDate);
        var frequencyInSeconds = this.getResourceFrequencyInSeconds(resourceConfig);

        return (diffInSeconds <= 0 || diffInSeconds >= frequencyInSeconds);
    },
    getResourceFrequencyInSeconds: function (config) {
        var resourceFrequency = config.frequency; //in format dd hh mm ss
        var values = (resourceFrequency + '').split(' ');
        if (values.length < 4) {
            throw "Invalid frequency to parse";
        }
        return (Number(values[0]) * 24 * 60 * 60 + Number(values[1]) * 60 * 60 + Number(values[1]) * 60 + Number(values[1]));
    },
    getResourcesRequiringUpdate: function () {
        var result = [];	
	var mgr = PFLEX.UpdateResource.Manager;
        PFLEX.$A(this.resources).pflexEach(function (resource) {
            var currentDate = PFLEX.DateUtil.getUTCNow();
            var lastCheckedDateStr = PFLEX.NativeResource.getResourceData(PFLEX.UpdateResource.config.resourceDataFilePath, resource.config.key + PFLEX.UpdateResource.config.lastCheckedDateSuffix);
            var lastCheckedDate = null;
            if (lastCheckedDateStr) {
		lastCheckedDate = PFLEX.DateUtil.parseTimestamp(lastCheckedDateStr);
            } else {
                lastCheckedDate = new Date(0);
            }	    
		if(typeof lastCheckedDate === 'undefined' || lastCheckedDate === null) {
			lastCheckedDate = new Date(0); 
		}	
            if (mgr.isUpdateRequired(currentDate, lastCheckedDate, resource.config)) {
                result.push(resource);
            }
        });
        return result;
    },
    checkForUpdates: function (adserverUrl, pid, callbackFn) {
        //prepare url using ad_server host and update params and pid
        //like <ad_mager_host>/update/resource/android/<pid>
        //it will return update json in format and make call to callback method
        var url = PFLEX.AppInfo.getAppInfoPflxAdUrl() + PFLEX.UpdateResource.config.updateUrlKey + pid;
        var headers = PFLEX.VersionHelper.getHeaders();
        PFLEX.receiveMessage(callbackFn, url, headers);
    }
}


// *** Common resourceUtil.js***

PFLEX.ResourceUtil = {
    getResource: function (resourceKey) {
        //check if isUpdateInProgress = true, return from old Location
	var resource = null;
        var updateResourceMgr = PFLEX.UpdateResource.Manager;
        for (key in updateResourceMgr.resources) {
            var resConfig = updateResourceMgr.resources[key].config;
            if (typeof resConfig !== "undefined" && resConfig !== null) {
                if (typeof resConfig.key !== "undefined" && resConfig.key !== null && resConfig.key === resourceKey) {
                    resource = updateResourceMgr.resources[key];
                    break;
                }
            }
        }
        if (resource === null || typeof resource === undefined) {
            return null;
        }
        var resourceContent = null;
        var isUpdateInProgress = resource.isUpdateInProgress()
        if (isUpdateInProgress == null || typeof isUpdateInProgress == undefined || isUpdateInProgress == "false") {
            var res_url = PFLEX.UpdateResource.config.resourceBasePath + '/' + resource.config.name;
            resourceContent = PFLEX.NativeResource.getResourceContent(res_url, "false", resourceKey);
        } //else return from original location
        else {
            var old_res_url = PFLEX.UpdateResource.config.resourceOldPath + '/' + resource.config.name;
            resourceContent = PFLEX.NativeResource.getResourceContent(old_res_url, "false", resourceKey);
        }
        //if not found at original location, try to fetch from assets folder
        if (resourceContent == null || typeof resourceContent === undefined) {
            var asset_res_url = '' + resource.config.name;
            resourceContent = PFLEX.NativeResource.getResourceContent(asset_res_url, "true", resourceKey);
        }
        return resourceContent;
    }
}
// *** Common Android.js***

/*
ActivityStorage have following methods:
- ActivityStorage.addRegistrationData(key, value);
- ActivityStorage.getRegistrationData(key);
- ActivityStorage.clearRegistrationData();
- ActivityStorage.hasValidRegistrationData();
- ActivityStorage.getAdditionalData(key);
- ActivityStorage.setAdditionalData(key,value);
*/

PFLEX.NativeRegistrationStorage = ActivityStorage;


/*
Activity have following methods defined:
        Activity.finishActivity();
        Activity.initializeData();
	Activity.isOnline();
	Activity.getLocale();
	Activity.getAppInfoJsonContent();
	Activity.logMessage(messageStr);
	Activity.cancelProgressDialog();
	Activity.getBasePath();
	Activity.asyncHttpRequest(url, isGet, headers);
	Activity.showKeyboard();
*/
PFLEX.Native = Activity;

PFLEX.NativeActions = {
	confirm: function(msg) {
		return confirm(msg);
    	},
	alert: function(msg) {
		return alert(msg);
    	},
};



PFLEX.NativeStorage = {
    	clearState: function() {
		PFLEX.StateStorage.clearState();
    	},
	supports_html5_storage: function() {
  		try {
    			return 'localStorage' in window && window['localStorage'] !== null;
  		} catch (e) {
    			return false;
  		}
  	},
        initialize: function() {
		if(typeof localStorage === "undefined") {
			PFLEX.console.log("Html5 localStorage is NOT found");
			localStorage = ActivityStorage;
		}
	},
	getItem: function(key) {
		this.initialize();
		return localStorage.getItem(key);
	},
	removeItem: function(key) {
		this.initialize();
		localStorage.removeItem(key);
	},
	setItem: function(key, value) {
		this.initialize();		
		localStorage.setItem(key, value);
	}
				
};

/*
ActivityDevice have following methods:
        ActivityDevice.getDeviceData(key);

	//0-undefined, 1-portrait, 2-landscape, 3-square
        ActivityDevice.getScreenOrientation();	

        ActivityDevice.getScreenWidth();	
        ActivityDevice.getScreenHeight();	
        ActivityDevice.getScreenDensity();	

	//0-undefined, 1-small, 2-normal, 3-large, 4-xlarge
        ActivityDevice.getScreenSize();	
*/
PFLEX.NativeDeviceData = ActivityDevice;

/*
ActivityVersionHelper have following methods:
		ActivityVersionHelper.getAppName();
		ActivityVersionHelper.getAppVersionCode();
		ActivityVersionHelper.getAppVersionName();
		ActivityVersionHelper.getOsVersion();
		ActivityVersionHelper.getUserAgent();
		ActivityVersionHelper.getLocale();
		ActivityVersionHelper.getPostalCode();	
*/
PFLEX.NativeVersionHelper = ActivityVersionHelper;
PFLEX.NativeVersion = {
	getOSType: function() {return "Android"; },
};

/*
ResourceUpdate have following methods:
		ResourceUpdate.setResourceData(resourceDataFile, resourceDataKey, resourceDataValue);
		ResourceUpdate.getResourceData(resourceDataFile, resourceDataKey);
		ResourceUpdate.getResourceContent(resourcePath, fromAssets, resourceKey);
		ResourceUpdate.updateResource(url, expectedChecksum, resourcePath, resourceTempPath, resourceOldPath, headers, extract, resKey);
}
*/
PFLEX.NativeResource = ResourceUpdate;


//Render placement & offers

PFLEX.renderJson = {};

PFLEX.renderJson.util = {};

PFLEX.renderJson.util.getFormData = function(formFields,placement){
		var formData = {};
			PFLEX.$A(formFields).pflexEach(function(field){
				formData[PFLEX.isDateField(field.elm) ? field.elm.getAttribute("name") : field.elm.name] = PFLEX.renderJson.event.convertToSignUpValue(field.elm.value, field.elm);
			});
	
		return formData;
	};
	
PFLEX.renderJson.util.addValidationClasses = function(input, classArray){
	PFLEX.$A(classArray).pflexEach(function(value){
		PFLEX.addClassName(input, PFLEX.Validation.tokenToClass[value.name]);
	});
};
	
PFLEX.renderJson.event = {};

PFLEX.renderJson.event.saveSignUpForm = function() {
};
	
PFLEX.renderJson.event.placementSubmit = function(formFields, multiofferSignupPage){
		var validation = new PFLEX.Validation();
		
		var form_values = {};
		for(var k=0;k<formFields.length;k++) {
			form_values[PFLEX.isDateField(formFields[k].elm) ? formFields[k].elm.getAttribute("name") : formFields[k].elm.name] = PFLEX.renderJson.event.convertToSignUpValue(formFields[k].elm.value, formFields[k].elm);
		}

		var valresult = validation.validate(formFields, form_values);
		if(typeof valresult !== "undefined" && valresult !== null && valresult !== "") {				
			if(PFLEX.NativeActions.alert(valresult)) {
				//do nothing
			}
		} else {
			var formData = PFLEX.renderJson.util.getFormData(formFields,this);
			var fields = PFLEX.AppInfo.getRegistrationFields();
			for( var key in fields ) {
				var signupField = PFLEX.SignUp.getField(key);
				signupField.setValue(formData[signupField.fieldName]);
			}
			if(multiofferSignupPage === true) {
				return true;
			} else {
				return PFLEX.SignupActions.confirm();
			}
		}	
		return false;
	};
	
PFLEX.renderJson.event.offerCheckboxChange = function(oid){
        var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid()) ;
		placement.toggleOfferState(oid);
	};
PFLEX.renderJson.event.updateDuplicateFields = function(elm, fieldName, fieldValue, formFields, offerId) {
	if(!offerId) {
		return;
	}
	var val = null;
	if(elm && PFLEX.isDateField(elm)) {
		val = PFLEX.renderJson.event.getDateFieldValue(elm);
	}
	for(var i=0; i< formFields.length; i++) {
		if(formFields[i].fieldType === 'custom') {
		} else { 
			if(formFields[i].fieldName == fieldName) {
				if (offerId != formFields[i].offerId) {
					if (PFLEX.isDateField(formFields[i].elm)) {
						PFLEX.renderJson.event.setDateFieldValue(val, formFields[i].elm);
					}
					else {
						formFields[i].elm.value = PFLEX.renderJson.event.convertToSignUpValue(fieldValue, formFields[i].elm);
					}
					PFLEX.renderJson.event.signUpFieldValueChange(fieldName, fieldValue, null, formFields[i].offerId);
				}
			}
		}
	}	
};

PFLEX.renderJson.event.convertToSignUpValue = function(fieldValue, elm) {
	if (PFLEX.isDateField(elm)) {
		var dateVal = PFLEX.renderJson.event.getDateFieldValue(elm);
		return dateVal.year+dateVal.month+(dateVal.date < 10 ? '0' + dateVal.date : dateVal.date);
	}
	return fieldValue;
};

PFLEX.renderJson.event.signUpFieldValueChange = function(fieldName, value, signupFieldFromEvent, offerId){
		var signupField = signupFieldFromEvent;
		if(offerId) {
			//console.log("Updating value for offerId ===== " + offerId + "  fieldName ==== " + fieldName);
			var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
			var offer = placement.offerRegistry[offerId];
			var offer_signup_field = offer.signUpFields[fieldName];
			offer_signup_field.setValue(value);
		} else {
			if(typeof signupFieldFromEvent === 'undefined' || signupFieldFromEvent === null) {
				var signupField = PFLEX.SignUp.getField(fieldName);
			}
			signupField.setValue(value);
		}		
	};
	
PFLEX.renderJson.util.renderField = function(field,fieldObj,id){
	var fieldType = fieldObj.type || "date"; // dates don't specify their type so we'll default to date 
	return PFLEX.renderJson.util.renderField[fieldType](field,fieldObj,id);
};

PFLEX.renderJson.util.renderField.text = function(field,fieldObj,id){
	return PFLEX.createEl({tag:"input",attr:{name:field, type:"text", pflexmessage:fieldObj.displayName, value: fieldObj.value, placeholder: fieldObj.displayName, title: fieldObj.displayName}, id:id});
};
	
PFLEX.renderJson.util.renderField.select = function(field,fieldObj,id){
	//if select type and options are undefined, create simple text field
	if(typeof fieldObj.options === 'undefined' || fieldObj.options === null || fieldObj.options.length === 0) {
		return PFLEX.renderJson.util.renderField.text(field,fieldObj,id);
	}


	var option, select = PFLEX.createEl({tag:"select",attr:{name:field, pflexmessage:fieldObj.displayName},id:id});
		
	for(option in fieldObj.options ){
		if(fieldObj.options[option]==fieldObj.value) {
			select.appendChild(PFLEX.createEl({tag:"option",attr:{value:fieldObj.options[option], selected: "selected"},html:option}));
		} else {
			select.appendChild(PFLEX.createEl({tag:"option",attr:{value:fieldObj.options[option]},html:option}));
		}	
	}
	return select;
};
	
	
PFLEX.renderJson.util.renderField.date = function(field,fieldObj,id){
			var dateValue = fieldObj.value;
			var date = null, month = null, year = null;
			if(dateValue) {
				var dateVal = PFLEX.DateUtil.parseDate(dateValue);
				date = dateVal.getDate();
				month = dateVal.getMonth()+1;
				year = dateVal.getFullYear();
				//console.log("date value === " + year+":"+month+":"+date);
			}
	        var i,
	                div = PFLEX.createEl({tag:"div","class":"dateField", attr:{pflexmessage:fieldObj.displayName,pflexFieldType:"date",name:field},id:id}),
	                monthSelect =  PFLEX.createEl({id:id + "_mm", tag:"select", "class":"month",attr:{name:field + "_mm",pflexFieldType:"month"}}),
	                daySelect =  PFLEX.createEl({id:id + "_dd", tag:"select", "class":"day",attr:{name:field + "_dd",pflexFieldType:"day"}}),
	                yearSelect =  PFLEX.createEl({id:id + "_yyyy", tag:"select", "class":"year",attr:{name:field + "_yyyy",pflexFieldType:"year"}}),
	                months = [{display:"- -", value:""},{display:"Jan", value:"01"},{display:"Feb", value:"02"},{display:"Mar", value:"03"},{display:"Apr", value:"04"},{display:"May", value:"05"},{display:"Jun", value:"06"},{display:"Jul", value:"07"},{display:"Aug", value:"08"},{display:"Sep", value:"09"},{display:"Oct", value:"10"},{display:"Nov", value:"11"},{display:"Dec", value:"12"}];
	       
	        PFLEX.$(months).pflexEach(function(monthVal,index){
				if (month && month == (monthVal.value*1)) {
					monthSelect.appendChild(PFLEX.createEl({
						tag: "option",
						attr: {
							value: monthVal.value,
							selected: "selected"
						},
						html: monthVal.display
					}));
				} else {
					monthSelect.appendChild(PFLEX.createEl({
						tag: "option",
						attr: {
							value: monthVal.value
						},
						html: monthVal.display
					}));
				}
	        });
	       
	        daySelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:""},html:"- -"}));
	        for(i=1;i<=31;i++){
				if (date && date == i) {
					daySelect.appendChild(PFLEX.createEl({
						tag: "option",
						attr: {
							value: i,
							selected: "selected"
						},
						html: i
					}));
				}
				else {
					daySelect.appendChild(PFLEX.createEl({
						tag: "option",
						attr: {
							value: i
						},
						html: i
					}));
				}
	        }
	       
	        var yearOptions = PFLEX.Options.getYearsOptions(fieldObj.optionType);
			yearSelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:""},html:"--"}));
			yearOptions.pflexEach(function(option) {
					if(''+option === ''+year) {
						yearSelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:option, selected: "selected"},html:option}));
					} else {
						yearSelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:option},html:option}));
					}	
			 	}
			);
	       
	        div.appendChild(monthSelect);
	        div.appendChild(daySelect);
	        div.appendChild(yearSelect);
	       
	        return div;
};
	

PFLEX.renderJson.renderPlacement = function(placement, isSignUpPage, signUpPageType, parentDiv, signUpRequired, multiofferSignupPage){
		var formFields = [], field, fieldDiv, label, input,
		
		divForCentering = PFLEX.createEl({tag:"div","class":"divForCentering"}),
		
		mainDiv = PFLEX.createEl({tag:"div","class":"placement",id:"mainDiv"}),
		titleDiv = PFLEX.createEl({tag:"div","class":"title",id:"titleDiv",html:""}),
		brandDiv = PFLEX.createEl({tag:"div","class":"brand",id:"brand"}),
		brandA = PFLEX.createEl({tag:"a","class":"brandAclass",attr: {href:"#"}, html:(signUpPageType !== "registration" ? PFLEX.StringHelper.get('signup_ads_by') : PFLEX.StringHelper.get('powered_by')) 
			+ " <img src='" + PFLEX.Native.getBasePath() + "/images/logo_sm24.png'/></a>"}),
		buttonDiv = PFLEX.createEl({tag:"div","class":"buttonContainer", id:"buttonContainer"}),
		form = PFLEX.createEl({tag:"form"}),

		header = PFLEX.createEl({tag:"div","class":"header",html:placement.headerText}),
		
		mainOfferDiv = PFLEX.createEl({tag:"div","class":"mainOffer"}),
		messagesDiv = PFLEX.createEl({tag:"div","class":"messages",id:"messages"}),
		errorDiv = PFLEX.createEl({tag:"div","class":isSignUpPage === true ? "errors_signup" : "errors" ,id:"errors"}),
		fieldset = PFLEX.createEl({tag:"fieldset", id:"pflx_fieldset"}),
		policy1 = PFLEX.createEl({tag:"p","class":"privacyclass",html:PFLEX.StringHelper.get('privacy1_text'),id:"policy1"}),
		policy2 = PFLEX.createEl({tag:"a","class":"privacyclass",attr: {href:"#"}, html:PFLEX.StringHelper.get('privacy2_text'),id:"policy2"}), /* utf-8 arrow -  \u2192 */
		confirmBtn = PFLEX.createEl({tag:"input", "class": "confirm",attr:{type:"button",value:isSignUpPage === true ? PFLEX.StringHelper.get('confirm_registration_text') : PFLEX.StringHelper.get('confirm_multioffer_text')}}), 
		skipBtn = PFLEX.createEl({tag:"input", "class": "skip",attr:{type:"button",value:isSignUpPage === true ? PFLEX.StringHelper.get('decline_registration_text') : PFLEX.StringHelper.get('decline_multioffer_text')}});

		var parentDivParent = parentDiv.parentNode;
		
		
		parentDivParent.appendChild(divForCentering);
		parentDivParent.removeChild(parentDiv);
		divForCentering.appendChild(parentDiv);
		//parentDivParent.appendChild(parentDiv);
		
		//parentDiv.appendChild(divForCentering);
		//divForCentering.appendChild(mainDiv);
		
		parentDiv.appendChild(mainDiv);
		mainDiv.appendChild(titleDiv);
		
		if(!isSignUpPage) {
			PFLEX.addClassName(buttonDiv, "buttonContainer_multioffer");			
		}		


		if(signUpPageType !== 'registration') {
			mainDiv.appendChild(header);
		}else {
			/*
			var headerImage = PFLEX.createEl({tag:"img",attr:{ 'src': PFLEX.AppInfo.getAppInfoPflxAdUrl()+ placement.headerImage}});
			var body = PFLEX.createEl({tag:"p","class":"body",html:placement.bodyText});
			mainDiv.appendChild(headerImage);
			mainDiv.appendChild(body);
			*/
			var regOfferDiv = PFLEX.createEl({tag:"div","class":"offer"});
    			var regOfferLabel = PFLEX.createEl({tag:"label",html:placement.headerText});
    			var regOfferBody = PFLEX.createEl({tag:"p",html:placement.bodyText});
    			var regOfferImg = PFLEX.createEl({tag:"img",attr:{ 'src': PFLEX.AppInfo.getAppInfoPflxAdUrl()+ placement.headerImage}});

			var regOffer = PFLEX.AppInfo.getRegistrationOffer();
    	
			if(typeof regOffer !== "undefined" && regOffer	!== null) {		
    				regOfferDiv.appendChild(regOfferImg);
    				regOfferDiv.appendChild(regOfferLabel);
    				regOfferDiv.appendChild(regOfferBody);			
			
				//var regOffer = PFLEX.renderJson.offerRender(placement.registrationOffer);
				mainDiv.appendChild(regOfferDiv)
			}		
		}

		mainDiv.appendChild(form);
		

		brandDiv.appendChild(brandA);
		mainDiv.appendChild(brandDiv);

		


		if(isSignUpPage === false) {
			//form.appendChild(errorDiv);
			form.appendChild(mainOfferDiv);
			document.title = PFLEX.StringHelper.get('multi_offer_title');
			titleDiv.innerHTML = PFLEX.StringHelper.get('multi_offer_title');
		}
		
		/*
		if(multiofferSignupPage === true) {
			PFLEX.addClassName(mainDiv,"multiofferSignupPage");
			
		}
		*/
		if(isSignUpPage === true || multiofferSignupPage === true) {

			var fields = PFLEX.AppInfo.getClusteredOrderedFields();

			for( var i=0; i < fields.length; i++ ) {
				var key = fields[i].fieldName;
				//check if its a clustered field
				if (fields[i].type == "composite") {
					var compositeFieldDiv = PFLEX.createEl({
						tag: "div",
						"class": "composite"
					});
					var compositeFields = [];
					for( var fieldKey in fields[i].fields) {
						compositeFields.push(PFLEX.extend(fields[i].fields[fieldKey], { fieldName : fieldKey } ));
					}
					var compositeFields = PFLEX.AppInfo.getSortedFields(compositeFields);
					for (var j = 0; j < compositeFields.length; j++) {
						var composite_field = PFLEX.SignUp.getField(compositeFields[j].fieldName);
						var composite_formField_rendered = PFLEX.renderJson.renderFormField(composite_field, errorDiv, messagesDiv, formFields);
						formFields.push({
							elm: composite_formField_rendered.input,
							fieldName: compositeFields[j].fieldName
						});
						compositeFieldDiv.appendChild(composite_formField_rendered.fieldDiv);
					}
					fieldset.appendChild(compositeFieldDiv);
				}
				else {
					var signupField = PFLEX.SignUp.getField(key);
					var formField_rendered = PFLEX.renderJson.renderFormField(signupField, errorDiv, messagesDiv, formFields);
					
					fieldset.appendChild(formField_rendered.fieldDiv);
					// mk - placing in obj for validation compatibility
					formFields.push({
						elm: formField_rendered.input,
						fieldName: key
					});
				}
			}

			//form.appendChild(errorDiv);
			form.appendChild(fieldset);

			if(isSignUpPage === true) {
				if(signUpPageType === "missinginfo") {
					document.title = PFLEX.StringHelper.get('missing_info_title');
					titleDiv.innerHTML = PFLEX.StringHelper.get('missing_info_title');					
				} else {
					document.title = PFLEX.VersionHelper.getAppName();
					titleDiv.innerHTML = PFLEX.VersionHelper.getAppName();
				}
			}
		}

		form.appendChild(messagesDiv);		

		form.appendChild(policy1);
		policy1.appendChild(policy2);
		
		buttonDiv.appendChild(skipBtn);
		buttonDiv.appendChild(confirmBtn);
		form.appendChild(buttonDiv);

		if(isSignUpPage === true) {
	        	form.onsubmit = function(){ return PFLEX.renderJson.event.placementSubmit(formFields);};
			// mk - hack to make button work in IE6
			confirmBtn.onclick = function(){ form.onsubmit(); return false; };		
			skipBtn.onclick = function(){ PFLEX.SignupActions.finishActivity(); };
			policy2.onclick = function(){ PFLEX.PrivacyActions.showPrivacyPage(true, multiofferSignupPage); };
			brandA.onclick = function(){ PFLEX.BaseActions.showSignupAdsPage(true, multiofferSignupPage); };

			
			if(typeof signUpRequired !== "undefined" && signUpRequired === "true") {
				skipBtn.disabled="true";
			}
		} else {
			form.onsubmit = function(){ return PFLEX.MultiofferActions.offersConfirmed(); };
			// mk - hack to make button work in IE6
			confirmBtn.onclick = function(){ form.onsubmit(); return false; };		
			skipBtn.onclick = function(){ PFLEX.MultiofferActions.finishActivity(); };
			policy2.onclick = function(){ PFLEX.PrivacyActions.showPrivacyPage(false, multiofferSignupPage); };
			brandA.onclick = function(){ PFLEX.BaseActions.showSignupAdsPage(false, multiofferSignupPage); };
			
			
			mainDiv.style.visibility = 'hidden';
		}
		
		if(multiofferSignupPage === true) {
			form.onsubmit = function(){ 
				var result = PFLEX.renderJson.event.placementSubmit(formFields, multiofferSignupPage);
				if(result === true) {
					return PFLEX.MultiofferSignupActions.offersConfirmed(); 
				}
			};
			// mk - hack to make button work in IE6
			confirmBtn.onclick = function(){ form.onsubmit(); return false; };
		}


		return { dom : mainDiv, form: form, mainOfferDiv : mainOfferDiv, config : placement, formFields : formFields};		
	};
	
    PFLEX.renderJson.renderFormField = function(signupField, errorDiv, messagesDiv, formFields, offerId){
		//load the error messages
    	/*var errorMessage = signupField.getErrorMessage();
    	if (typeof errorMessage !== "undefined" && errorMessage !== "undefined") {
        	if (errorMessage !== "null" && errorMessage !== "") {
            	errorDiv.innerHTML += errorMessage + "<br>";
            }
        };*/
		
		if(signupField.type == 'select' && (!signupField.hasOwnProperty('options') || typeof signupField.options === 'undefined')) {
			signupField.options = PFLEX.Options[signupField.optionType];
		}

		var countryCode = 'US';
                if(signupField.optionType === 'state') {
			//check if there is any form field country, if yes select value of it
                        //else fetch it from server
			
			var countryFieldName = PFLEX.AppInfo.getCountryFieldName();
			var countryElm = null;
			for(var i=0; i< formFields.length; i++) {		 
				if(formFields[i].fieldName == countryFieldName) {
					countryElm = formFields[i].elm;
				}
			}

			if(typeof countryElm !== 'undefined' && countryElm !== null)  {
				countryCode = countrySignupField.value;	
			} else {
				countryCode = PFLEX.StorageUtil.getRegistrationData(countryFieldName);
			}
			if(typeof countryCode === 'undefined' || countryCode === null)  {
				countryCode = 'US';
			}
			signupField.options = PFLEX.Options['state'][countryCode];		
		}
		
		//check if any value is populated from device
		var valuePopulatedFromDevice = signupField.isPopulatedFromDevice();
		if(typeof valuePopulatedFromDevice !== "undefined" && valuePopulatedFromDevice === "true") {
			messagesDiv.innerHTML = ''+PFLEX.StringHelper.get('fields_prepopulated');	
		}				

        fieldDiv = PFLEX.createEl({tag: "div", "class":signupField.fieldName});
		var suffix = offerId ? "_offer"+offerId: "";
        label = PFLEX.createEl({tag: "label", id:"label_"+signupField.fieldName + suffix, html: signupField.displayName, attr: { "for": "pflex_" + signupField.fieldName + suffix} });     
        input = PFLEX.renderJson.util.renderField(signupField.fieldName + suffix, signupField, "pflex_" + signupField.fieldName + suffix);
        
        var countryFieldName = PFLEX.AppInfo.getCountryFieldName();
        var postalCodeFieldName = PFLEX.AppInfo.getPostalCodeFieldName();
        
      //If country name is not US, change postal code displayName to "Postal Code" (instead of "Zip")
		if (signupField.fieldName === countryFieldName) {
			var zipElem = document.getElementById("label_" + postalCodeFieldName + suffix);
			var placeHolderElem = document.getElementById("pflex_"+postalCodeFieldName + suffix);
			if(typeof zipElem !== 'undefined' && zipElem !== null) {
				 if(signupField.value === "US") {
					if(placeHolderElem)placeHolderElem.placeholder = "Zip";
				     zipElem.innerHTML = 'Zip';
				 }
				 else {
					 if(placeHolderElem)placeHolderElem.placeholder = "Postal Code";
					 zipElem.innerHTML = 'Postal Code';
				 }				 
			}
		}		
        
		if (signupField.fieldName === postalCodeFieldName) {
			var countrySignupField = PFLEX.SignUp.getField(countryFieldName);
			if(typeof countrySignupField !== 'undefined' && countrySignupField !== null)  {
				 if(countrySignupField.value === "US") {
					 input.placeholder = "Zip";
					 label.innerHTML = 'Zip';
				 }
				else {
					input.placeholder = "Postal Code";
					label.innerHTML = 'Postal Code';
				}
			}
		}
		if (!PFLEX.isDateField(input)) {
			PFLEX.renderJson.setEvents(signupField, errorDiv, messagesDiv, formFields, offerId, input);
		} else {
			var yearInput = input.children[input.id+'_yyyy'];
			var monthInput = input.children[input.id+'_mm'];
			var dateInput = input.children[input.id+'_dd'];
			PFLEX.renderJson.setEvents(signupField, errorDiv, messagesDiv, formFields, offerId, yearInput, input);
			PFLEX.renderJson.setEvents(signupField, errorDiv, messagesDiv, formFields, offerId, monthInput, input);
			PFLEX.renderJson.setEvents(signupField, errorDiv, messagesDiv, formFields, offerId, dateInput, input);
		}
        
        PFLEX.renderJson.util.addValidationClasses(input, signupField.validations);
        
        fieldDiv.appendChild(label);
        fieldDiv.appendChild(input);

	//fix label for telephone number
	if(signupField.isPhoneField()) {
		var phoneData = PFLEX.PhoneUtil.parsePhone(signupField.value);
		input.value = phoneData.phonenumber;
		//label.innerHTML = label.innerHTML+ '  '+phoneData.isd;		
		
		//Add regulatory message
    	var regulatoryMessage = PFLEX.StringHelper.get('regulatory_phone');
    	if(regulatoryMessage) {        	
    		disclaimerLabel = PFLEX.createEl({tag: "label", id:"label_disclaimer", html: regulatoryMessage});
    	}
    	fieldDiv.appendChild(disclaimerLabel);
	}
	
	//for state incase we are trying to get options from server, replace text input field with dropdown
	if(signupField.optionType === 'state') {
		if(typeof signupField.options  === 'undefined' || signupField.options === null)  {
			//fetch from server
			var callbackFunction = function (jsonStr) {
				if(typeof jsonStr !== 'undefined' && jsonStr !== null && jsonStr !== '' && jsonStr !== 'undefined') {
					var optionsJsonObj = JSON.parse(jsonStr);
					signupField.options = optionsJsonObj;
					fieldDiv.removeChild(input);
					input = PFLEX.renderJson.util.renderField(signupField.fieldName + suffix, signupField, "pflex_" + signupField.fieldName + suffix);
					PFLEX.renderJson.setEvents(signupField, errorDiv, messagesDiv, formFields, offerId, input);	
        
				        PFLEX.renderJson.util.addValidationClasses(input, signupField.validations);
					fieldDiv.appendChild(input);
				}

			};
			PFLEX.BaseActions.getStateList(countryCode, callbackFunction);
		}
	}
	return {fieldDiv : fieldDiv, input : input};
    }


PFLEX.renderJson.setEvents = function(signupField, errorDiv, messagesDiv, formFields, offerId, input, parentElm) {
	var countryFieldName = PFLEX.AppInfo.getCountryFieldName();
        var postalCodeFieldName = PFLEX.AppInfo.getPostalCodeFieldName();	
		input.onchange = function(e){
        	if (e.target.name === countryFieldName) {
        		var zipElem = document.getElementById("label_" + postalCodeFieldName);
        		var placeHolderElem = document.getElementById("pflex_"+postalCodeFieldName);
        		
        		if(typeof zipElem !== 'undefined' && zipElem !== null) {
	        			if (e.target.value === "US") {
	        				if(placeHolderElem)placeHolderElem.placeholder = "Zip";
	        				zipElem.innerHTML = 'Zip';
	        			}
	        			else {
	        				if(placeHolderElem)placeHolderElem.placeholder = "Postal Code";
	        				zipElem.innerHTML = 'Postal Code';
	        			}
				}
        	}
			var fieldValue = PFLEX.renderJson.event.convertToSignUpValue(parentElm ? null : e.target.value, parentElm ? parentElm : e.target);
			PFLEX.renderJson.event.updateDuplicateFields(parentElm, signupField.fieldName, fieldValue, formFields, offerId);
            return PFLEX.renderJson.event.signUpFieldValueChange(signupField.fieldName, fieldValue, signupField, offerId);
        };
        input.onkeydown = function tabE(e){
			var fieldValue = PFLEX.renderJson.event.convertToSignUpValue(parentElm ? null : e.target.value, parentElm ? parentElm : e.target);
			PFLEX.renderJson.event.updateDuplicateFields(parentElm, signupField.fieldName, fieldValue, formFields, offerId);
            return PFLEX.renderJson.event.signUpFieldValueChange(signupField.fieldName, fieldValue, signupField, offerId);
        };
        input.onkeyup = function tabE(e){
			var fieldValue = PFLEX.renderJson.event.convertToSignUpValue(parentElm ? null : e.target.value, parentElm ? parentElm : e.target);
			PFLEX.renderJson.event.updateDuplicateFields(parentElm, signupField.fieldName, fieldValue, formFields, offerId);
            return PFLEX.renderJson.event.signUpFieldValueChange(signupField.fieldName, fieldValue, signupField, offerId);
        };
        
        input.onkeypress = function tabE(e){
			var fieldValue = PFLEX.renderJson.event.convertToSignUpValue(parentElm ? null : e.target.value, parentElm ? parentElm : e.target);
			PFLEX.renderJson.event.updateDuplicateFields(parentElm, signupField.fieldName, fieldValue, formFields, offerId);
            PFLEX.renderJson.event.signUpFieldValueChange(signupField.fieldName, fieldValue, signupField, offerId);
            var e = (typeof window.event != 'undefined') ? window.event : e;// IE : Moz
            if (e.keyCode == 13) {
                for (var i = 0; i < formFields.length; i++) {
                    if (e.target == formFields[i].elm) {
                        var ele = (i == formFields.length - 1) ? formFields[i].elm : formFields[i + 1].elm;
                        
                        //fix for v1.5 and v1.6 - remove hint
                        if (typeof ele.value === "undefined" || ele.value === null || ele.value === '') {
                            ele.value = '';
                        }
                        if (ele.nodeName === "INPUT" && ele.type === "text" && i !== formFields.length - 1) {
                        	ele.focus();
                        	Activity.showKeyboard();
                        }
                    }
                }
                return false;
            }
        };
        
        
        
        input.onclick = function tabE(e){
            var e = (typeof window.event != 'undefined') ? window.event : e;// IE : Moz
            for (var i = 0; i < formFields.length; i++) {
                if (e.target == formFields[i].elm) {
                    var ele = formFields[i].elm;
                    //fix for v1.5 and v1.6 - remove hint
                    if (typeof ele.value === "undefined" || ele.value === null || ele.value === '') {
                        ele.value = '';
                    }
                    ele.focus();
                }
            }
            return false;
            
        };
        
        input.onfocus = function tabE(e){
            var e = (typeof window.event != 'undefined') ? window.event : e;// IE : Moz
            for (var i = 0; i < formFields.length; i++) {
                if (e.target == formFields[i].elm) {
                    var ele = formFields[i].elm;
                    //fix for v1.5 and v1.6 - remove hint
                    if (typeof ele.value === "undefined" || ele.value === null || ele.value === '') {
                        ele.value = '';
                    }
                }
            }
            return false;
            
        };      
}


PFLEX.renderJson.offerRender = function(offer,index,isOfferSignUpPage, hasOfferFieldsToSignUp){
	var div = PFLEX.createEl({tag: "div","class": "offer offer" + index + (hasOfferFieldsToSignUp ?" offer_requiresignup" : "")});
	var headerText = isOfferSignUpPage ? PFLEX.format( (hasOfferFieldsToSignUp ? PFLEX.StringHelper.get('offer_signup_field_headerformat') : 
						PFLEX.StringHelper.get('offer_signup_nofield_headerformat') ), offer.getHeaderText()) :offer.getHeaderText();
    var label = PFLEX.createEl({tag:"label","class":(hasOfferFieldsToSignUp ? "withfields":"withoutfields") , html:headerText,attr:{ "for": offer.getOid()}});
    input = PFLEX.createEl({tag:"input","class":"checkbox",attr:{type:"checkbox"}, id:"chkbox_"+offer.getOid(), prop: {checked: offer.isSelected() === 'true' ? true : false}}),
    checkboxDiv = PFLEX.createEl({tag:"div","class":"checkboxDiv"}),
    body = PFLEX.createEl({tag:"p",html: offer.getBodyText()}),
    img = PFLEX.createEl({tag:"img",attr:{ 'src': PFLEX.AppInfo.getAppInfoPflxAdUrl()+ offer.getImagePath()}}),
    hr = PFLEX.createEl({tag:'img','class':'hr',attr:{'src':PFLEX.Native.getBasePath()+'/images/hr.jpg'}});
	if (!isOfferSignUpPage) {
		div.appendChild(input);
		div.appendChild(checkboxDiv);
	}
    div.appendChild(img);
    div.appendChild(label);
	if (!isOfferSignUpPage) {
		div.appendChild(body);
		div.appendChild(hr);
	}
    if(!isOfferSignUpPage && offer.isSelected() === 'true') {
		PFLEX.addClassName(div,"selected");	
    }
    
    var thisOffer = offer;
	if (!isOfferSignUpPage) {
		div.onclick = function(){
			PFLEX.renderJson.event.offerCheckboxChange(thisOffer.getOid());
			var chkbox = document.getElementById("chkbox_" + thisOffer.getOid());
			if (chkbox.checked) {
				PFLEX.addClassName(div, "selected");
			}
			else {
				PFLEX.removeClassName(div, "selected");
			}
			
			return;
		};
	}
    
    return { dom : div, hr: hr, label:label }; 
};

PFLEX.renderJson.event.offersLoaded = function() {
	document.getElementById('mainDiv').style.visibility = 'visible'; 
}; 

PFLEX.renderJson.Offer = function(offer,config,index){

    var newOffer = PFLEX.renderJson.offerRender(offer,index);

	newOffer.trackImpression = function(){
		var url = PFLEX.stats + "/imp/offer/" + offer.oid;
		if(PFLEX.isString(config.creativeId)){ url += "/" + config.creativeId; }
		if(PFLEX.isString(config.subSourceId)){ url += "/" + config.subSourceId; }
		
		PFLEX.renderPixel(url,true);
	};
    
    return newOffer;
};


PFLEX.renderJson.Placement = function(config, signUpPageType, parentDiv, signUpRequired, multiofferSignupPage){

        var regOffer = 	PFLEX.AppInfo.getOffer();        

	// mk - setting defaults
	config = PFLEX.extend({
		headerText : "",
		headerImage : signUpPageType == 'registration' && (typeof regOffer !== "undefined" && regOffer !== null) ? regOffer.getImagePath(): "",
		bodyText : signUpPageType == 'registration' && (typeof regOffer !== "undefined" && regOffer !== null) ? regOffer.getBodyText(): "",
		privacyPolicyUrl : "http://www.pflexads.com/privacy_policy.html"
		}, config);
		
	var placement = {
		config : config,
		offers : [],
		submitURL : "@appweb_base_url@/bpost",
		setOffers : function(offerArray){
			PFLEX.$A(offerArray).pflexEach(function(offer,index){
				var parentDiv, newOffer;
				newOffer = PFLEX.renderJson.Offer(offer,this,index);
				this.offers.push(newOffer);
				newOffer.trackImpression();
				this.nodes.mainOfferDiv.appendChild(newOffer.dom);
			},this);
			//send event that offers are loaded
			PFLEX.renderJson.event.offersLoaded();
			
		},
		trackImpression : function() {
			var url = PFLEX.stats + "/imp/placement/" + this.config.pid;
			if(PFLEX.isString(this.config.subSourceId)){ url += "/" + this.config.subSourceId; }
			PFLEX.renderPixel(url,true);
		}
	};
	if(signUpPageType !== null) {
		if(signUpPageType == 'registration') {
			if (typeof regOffer !== "undefined" && regOffer !== null) {
				config.headerText = regOffer.getHeaderText();
			}
		} else {
			config.headerText = PFLEX.StringHelper.get('missing_info_header');
		}
	}

	placement.nodes = PFLEX.renderJson.renderPlacement(config, signUpPageType != null, signUpPageType, parentDiv, signUpRequired, multiofferSignupPage);

	placement.trackImpression();
	
	
	return placement;
};


PFLEX.renderJson.thankssAndFinishRender = function() {
	var mainDiv = document.getElementById('main'),
	contentDiv = PFLEX.createEl({tag:"div","class":"thanksAndFinish"}),
	buttonDiv = PFLEX.createEl({tag:"div"}),
	closeBtn = PFLEX.createEl({tag:"button",attr:{type:"submit"},html:PFLEX.StringHelper.get('return_to_application')});
	closeBtn.onclick = function(){ PFLEX.Native.finishActivity(); };

	PFLEX.addClassName(mainDiv,"thanks");
	
	contentDiv.innerHTML = ''+PFLEX.StringHelper.get('thank_you_for_registering');
	buttonDiv.appendChild(closeBtn);

	while (mainDiv.hasChildNodes()) {
	    mainDiv.removeChild(mainDiv.lastChild);
	}

	mainDiv.appendChild(contentDiv);
	mainDiv.appendChild(buttonDiv);

};

PFLEX.renderJson.renderOffersForSignUp = function(offers) {
		//console.log("offers signup================="+JSON.stringify(offers));
	if(offers == null || typeof offers == 'undefined' || offers.length == 0) {
		console.log("Invalid offers to render for offers signup");
	}
	var parentDiv = document.getElementById('main');
	
	var mainDiv = PFLEX.createEl({tag:"div","class":"placement",id:"mainDiv"}),		
	divForCentering = PFLEX.createEl({tag:"div","class":"divForCentering"}),
	mainOfferDiv = PFLEX.createEl({tag:"div","class":"mainOffer"}),
	titleDiv = PFLEX.createEl({tag:"div","class":"title",id:"titleDiv",html:""}),
	header = PFLEX.createEl({tag:"div","class":"header",html:PFLEX.StringHelper.get('offer_signup_header')}),
	messagesDiv = PFLEX.createEl({tag:"div","class":"messages",id:"messages"}),
	errorDiv = PFLEX.createEl({tag:"div","class":"errors" ,id:"errors"}),
	brandDiv = PFLEX.createEl({tag:"div","class":"brand",id:"brand"}),
    brandA = PFLEX.createEl({tag:"a","class":"brandAclass",attr: {href:"#"}, html:PFLEX.StringHelper.get('signup_ads_by') + " <img src='" + PFLEX.Native.getBasePath() + "/images/logo_sm24.png'/></a>"}),
	policy1 = PFLEX.createEl({tag:"p","class":"privacyclass",html:PFLEX.StringHelper.get('privacy1_text'),id:"policy1"}),
	policy2 = PFLEX.createEl({tag:"a","class":"privacyclass",attr: {href:"#"}, html:PFLEX.StringHelper.get('privacy2_text'),id:"policy2"}), /* utf-8 arrow -  \u2192 */
	buttonDiv = PFLEX.createEl({tag:"div","class":"buttonContainer buttonContainer_multioffer", id:"buttonContainer"}),
	confirmBtn = PFLEX.createEl({tag:"input", "class": "confirm",attr:{type:"button",value:PFLEX.StringHelper.get('confirm_multioffer_text')}}), 
	skipBtn = PFLEX.createEl({tag:"input", "class": "skip",attr:{type:"button",value: PFLEX.StringHelper.get('decline_multioffer_text')}}),
	header_hr = PFLEX.createEl({tag:'img','class':'header_hr',attr:{'src':PFLEX.Native.getBasePath()+'/images/hr.jpg'}});

	var parentDivParent = parentDiv.parentNode;		
		parentDivParent.appendChild(divForCentering);
		parentDivParent.removeChild(parentDiv);
		divForCentering.appendChild(parentDiv);
		
	parentDiv.appendChild(mainDiv);
	mainDiv.appendChild(titleDiv);
	mainDiv.appendChild(header);
	mainDiv.appendChild(header_hr);
	mainDiv.appendChild(mainOfferDiv);
	titleDiv.innerHTML = PFLEX.StringHelper.get('missing_info_title');

	var formFields = [];
	for (var index = 0; index < offers.length; index++) {
		var offer = offers[index];
		var fields = offer.getFieldsForRendering();
		var hasOfferFieldsToSignUp = (fields != null && fields != 'undefined' && fields.length > 0);
		var newOffer = PFLEX.renderJson.offerRender(offer, index,true, hasOfferFieldsToSignUp);
		mainOfferDiv.appendChild(newOffer.dom);
		if (hasOfferFieldsToSignUp) {
			//console.log("offers signup fields================="+JSON.stringify(fields));
			var fieldset = PFLEX.createEl({
				tag: "fieldset",
				id: "pflx_fieldset"
			});
			for (var i = 0; i < fields.length; i++) {
				var key = fields[i].fieldName;
				//check if its a clustered field
				if (fields[i].type == "composite") {
					var compositeFieldDiv = PFLEX.createEl({
						tag: "div",
						"class": "composite"
					});
					var compositeFields = [];
					for (var fieldKey in fields[i].fields) {
						compositeFields.push(PFLEX.extend(fields[i].fields[fieldKey], {
							fieldName: fieldKey
						}));
					}
					var compositeFields = offer.getSortedFields(compositeFields);
					for (var j = 0; j < compositeFields.length; j++) {
						var composite_field = offer.signUpFields[compositeFields[j].fieldName];
						var composite_formField_rendered = PFLEX.renderJson.renderFormField(composite_field, errorDiv, messagesDiv, formFields, offer.getOid());
						formFields.push({
							elm: composite_formField_rendered.input,
							fieldName: compositeFields[j].fieldName,
							offerId: offer.getOid()
						});
						compositeFieldDiv.appendChild(composite_formField_rendered.fieldDiv);
					}
					fieldset.appendChild(compositeFieldDiv);
				}
				else {
					var signupField = offer.signUpFields[key];
					var formField_rendered = PFLEX.renderJson.renderFormField(signupField, errorDiv, messagesDiv, formFields, offer.getOid());
					fieldset.appendChild(formField_rendered.fieldDiv);
					// mk - placing in obj for validation compatibility
					formFields.push({
						elm: formField_rendered.input,
						fieldName: key,
						offerId: offer.getOid(),
						fieldType: signupField.fieldType
					});
				}
			}
			newOffer.dom.appendChild(fieldset);
		}
		newOffer.dom.appendChild(newOffer.hr);	
	}
	mainDiv.appendChild(policy1);
	policy1.appendChild(policy2);
		
	buttonDiv.appendChild(skipBtn);
	buttonDiv.appendChild(confirmBtn);
	mainDiv.appendChild(buttonDiv);
	
	brandDiv.appendChild(brandA);
	mainDiv.appendChild(brandDiv);
		
	confirmBtn.onclick = function(){ PFLEX.renderJson.event.offerSignUpSubmit(formFields); return false; };		
	skipBtn.onclick = function(){ PFLEX.OfferSignupActions.finishActivity(); };
	policy2.onclick = function(){ PFLEX.PrivacyActions.showPrivacyPage(false, false, true); };
	brandA.onclick = function(){ PFLEX.BaseActions.showSignupAdsPage(false, false, true); };
	
};

PFLEX.renderJson.event.getDateFieldValue = function(elm) {
	var year = elm.children[elm.id+'_yyyy'].value;
	var month = elm.children[elm.id+'_mm'].value;
	var date = elm.children[elm.id+'_dd'].value;
	return {
		year: year,
		month: month,
		date: date
	};
};

PFLEX.renderJson.event.setDateFieldValue = function(dateVal, elm) {
	elm.children[elm.id+'_yyyy'].value = dateVal.year;
	elm.children[elm.id+'_mm'].value = dateVal.month;
	elm.children[elm.id+'_dd'].value = dateVal.date;
};

PFLEX.renderJson.event.offerSignUpSubmit = function(formFields){
		var validation = new PFLEX.Validation();
		
		var form_values = {};
		for(var k=0;k<formFields.length;k++) {
			form_values[formFields[k].fieldName] = PFLEX.renderJson.event.convertToSignUpValue(formFields[k].elm.value, formFields[k].elm);
		}
		var valresult = validation.validate(formFields, form_values);
		if(typeof valresult !== "undefined" && valresult !== null && valresult !== "" && valresult !== "true" && valresult !== true) {				
			if(PFLEX.NativeActions.alert(valresult)) {
				//do nothing
			}
		} else {
			var formData = PFLEX.renderJson.util.getFormData(formFields);
			var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
			for(var i=0;i<formFields.length;i++) {
				var offer = placement.offerRegistry[formFields[i].offerId];
				var offer_signup_field = offer.signUpFields[formFields[i].fieldName];
				var valueToSet = formData[PFLEX.isDateField(formFields[i].elm) ? formFields[i].elm.getAttribute("name") : formFields[i].elm.name];
				offer_signup_field.setValue(''+valueToSet);
			}
			return PFLEX.OfferSignupActions.offersConfirmed();
		}	
		return false;
};
// **** Common Action.js ***

PFLEX.BaseActions = PFLEX.create();

PFLEX.BaseActions.prototype = {

init: function () {	
}
};

PFLEX.extend(PFLEX.BaseActions, {

currentPage: {},
setCurrentPage: function(pageName) {
	this.currentPage[0] = pageName;
},
getCurrentPage: function() {
	return this.currentPage[0];
},
resolveCurrentPage: function(pageName) {
	this.setCurrentPage(''+pageName);
	var savedPage = PFLEX.PageTransitionState.getSavedPage();

	PFLEX.PageTransitionState.clearSavedPage();
    if(typeof savedPage !== "undefined" && savedPage !== null && (''+savedPage !== ''+pageName)) { 		
		this.setCurrentPage(savedPage);
		location.replace('./'+this.getCurrentPage());
	}
},
leadsUrl: function () {
	return "https://app.pontiflex.com/bpost";
},
statsUrl: function () {
	return "http://stats.pflexads.com";
},
pendingOffer: {},
setPendingOffer: function(pendingOffer) {
	PFLEX.BaseActions.pendingOffer[0] = pendingOffer;
},
getPendingOffer: function() {
	return PFLEX.BaseActions.pendingOffer[0];
},
postAcceptedOffer: function (offer) {
	
	if(typeof offer === "undefined" || offer === null)
		return;

	var offerid = offer.getOid();
	if(typeof offerid === "undefined" || offerid === null || offerid === "-1")
		return;


	var isOnlineValue = ''+PFLEX.Native.isOnline();
	if(isOnlineValue === "true") {
		var url = PFLEX.BaseActions.leadsUrl();
		url = url + "?";
		var counter = "0";
		//set placement level reg fields                
		var fieldNames = PFLEX.AppInfo.getOrderedFieldNames();
		for(var key in fieldNames) {
			if(counter !== "0") {
				url = url +"&";
			}			
			counter = counter + 1;
			var fieldNameFromKey = fieldNames[key];
			url = url + encodeURIComponent(fieldNameFromKey) + "="+encodeURIComponent(PFLEX.StorageUtil.getRegistrationData(fieldNameFromKey));			
		}
                //set offer level signup fields
                for(var signupKey in offer.signUpFields) {
			
			var signupField = offer.signUpFields[signupKey];
               	        var signupFieldKey = signupField.fieldName;
               	        var signupFieldValue = signupField.value;
               	        if(typeof signupFieldKey !== "undefined" && signupFieldKey !== null) {
				if(signupField.isPhoneField()) {
					var phoneDate = PFLEX.PhoneUtil.parsePhone(signupField.value);
					signupField.value = PFLEX.PhoneUtil.format(phoneDate);
					signupFieldValue = signupField.value;
				} 

				if(this.isNotPresent(signupFieldKey, fieldNames)) {
					if(counter !== "0") {
						url = url +"&";
					}			
					counter = counter + 1;
					url = url + encodeURIComponent(signupFieldKey) + "="+encodeURIComponent(signupFieldValue);	
				}
			}   	
		}

		url = url + "&"+"pontiflexdealid="+encodeURI(offer.getDealId());
		url = url + "&"+"pflexoid="+encodeURI(offer.getOid());
		url = url + "&"+"pflexcrid="+encodeURI(offer.getCreativeId());
		url = url + "&"+"pflexsubsrcid="+encodeURI(PFLEX.AppInfo.getSubSourceId());

		var callbackFunction = function (jsonStr) {

		};
		var headers = JSON.stringify(PFLEX.VersionHelper.getHeaders());
		PFLEX.Native.asyncHttpRequest(url, true, headers);

	} else {
		PFLEX.BaseActions.setPendingOffer(offer);
		PFLEX.StateStorage.savePendingOffer();
	}
},
isNotPresent: function(signupFieldKey, keylist) {
	for(var key in keylist) {
		var fieldNameFromKey = keylist[key];
		if(''+signupFieldKey === ''+fieldNameFromKey) {
			return false;
		}
	}
	return true;	
},
handlePendingOffer: function() {
	var pendingOfferFromState = PFLEX.StateStorage.getPendingOffer();
	if(typeof pendingOfferFromState === "undefined" || pendingOfferFromState === null) {
		return;
	}
	var isOnlineValue = ''+PFLEX.Native.isOnline();
	if(isOnlineValue === "true") {
		PFLEX.BaseActions.postAcceptedOffer(pendingOfferFromState);
		PFLEX.StateStorage.clearPendingOffer();
	}
},
trackAppImpression: function () {
	var url = PFLEX.BaseActions.statsUrl() + "/imp/placement/" + PFLEX.AppInfo.getPid();
	var subsourceId = PFLEX.AppInfo.getSubSourceId();
	var timeInMills = ''+new Date().getTime();
	if(typeof subsourceId !== "undefined" && subsourceId !== null && subsourceId !== "") {
		url = url +"/"+encodeURI(subsourceId);
	}
	url = url + "?"+ timeInMills;

	var callbackFunction = function (jsonStr) {
	};

	var headers = PFLEX.VersionHelper.getHeaders();
	if(PFLEX.Native.isOnline()) {
		PFLEX.receiveMessage(callbackFunction, url, headers); 
	}
},

trackImpressions: function () {
	var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
	var offersList = placement.getOffers();
	for(var key in offersList) {
		var o = offersList[key];
		var selected = o.selected;
		if(typeof selected !== "undefined" && selected !== null) {
			PFLEX.BaseActions.trackImpression(o);
		}
	}
},

trackImpression: function (offer) {
	if(typeof offer === "undefined" || offer === null)
		return;

	var offerid = offer.getOid();
	if(typeof offerid === "undefined" || offerid === null || offerid === "-1")
		return;

	var url = PFLEX.BaseActions.statsUrl() + "/imp/offer/" + encodeURI(offer.getOid()) +"/" + encodeURI(offer.getCreativeId());
	var subsourceId = PFLEX.AppInfo.getSubSourceId();
	var timeInMills = ''+new Date().getTime();
	if(typeof subsourceId !== "undefined" && subsourceId !== null && subsourceId !== "") {
		url = url +"/"+encodeURI(subsourceId);
	}
	url = url + "?"+ timeInMills;

	var callbackFunction = function (jsonStr) {
	};

	var headers = PFLEX.VersionHelper.getHeaders();
	PFLEX.receiveMessage(callbackFunction, url, headers); 

},
thanksAndFinish: function() {
	if(PFLEX.NativeActions.confirm(PFLEX.StringHelper.get('thank_you_for_registering'))) {
		PFLEX.Native.finishActivity();
	} else {
		//do nothing
	}
},
getSignupAdsUrl: function() {
	return "http://www.pflexads.com";
},

showSignupAdsPage : function(signUpCase, multiofferSignupPage, offerSignupPage) {

	var signupAdsUrl = this.getSignupAdsUrl();

		var privacyPageConfig = new PFLEX.PrivacyPageConfig(signupAdsUrl, signUpCase, 'registration', '', 'privacy', PFLEX.StringHelper.get('signup_ads_title'));
		if(signUpCase || multiofferSignupPage) {
			PFLEX.StorageUtil.saveRegFormState();
			var signupPageType = PFLEX.SignUp.getPageType();
			if(typeof signupPageType !== "undefined" && signupPageType === "missinginfo") {
				PFLEX.PageTransitionState.setSignupPageType('missinginfo');
				privacyPageConfig.setSignUpPageType(signupPageType);
			}

		}
		if(offerSignupPage) {
			privacyPageConfig.setSignUpPageType('offer_signup');
		}
		if(!signUpCase) {
			//multioffer page, store urrent offers state
			PFLEX.StorageUtil.saveOffersState();
		}
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		//load privacy page, this will load url set in PrivacyPageConfig which is signupAdsUrl
		location.replace('./privacy.html');
},
getOffersCount: function() {
	if(this.isTabDevice()) {
		return 6;
	}
	return 3;
},
isTabDevice: function() {
	var screenSize = PFLEX.NativeDeviceData.getScreenSize();
	if(screenSize === 4) {
		return true;
	}
	return false;
},
setScreenOrientationAndSize: function() {

	var screenOrientation = PFLEX.NativeDeviceData.getScreenOrientation();
	//console.log('============screenOrientation=============='+screenOrientation);	

	//0-undefined, 1-portrait, 2-landscape, 3-square
	if(screenOrientation === 2) {//landscape
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"landscape");
	} else {
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"portrait");
	}

	var screenSize = PFLEX.NativeDeviceData.getScreenSize();
	//console.log('============screenSize=============='+screenSize);
	if(screenSize === 1) {//small
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"small");
	}
	if(screenSize === 2) {//normal
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"normal");
	}
	if(screenSize === 3) {//large
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"large");
	}
	if(screenSize === 4) {//xlarge
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"xlarge");

		if(screenOrientation === 2) {//landscape
			PFLEX.addClassName(document.getElementsByTagName("body")[0],"landscape");
		} else {
			PFLEX.addClassName(document.getElementsByTagName("body")[0],"portrait");
		}
	}
			
	
},
postAcceptedOffers: function(offersList) {
	var result = true;
	for(var key in offersList) {
		var o = offersList[key];
		var selected = o.selected;
		if(typeof selected !== "undefined" && selected !== null && selected === "true") {
			var offerAdditionalFields = o.getOfferAdditionalFields();
                        if(typeof offerAdditionalFields !== 'undefined' && offerAdditionalFields !== null && offerAdditionalFields.length>0) {
				if(o.hasValidSignUpFields()) {
					if(PFLEX.SignUp.showAdditionalFieldsForm === 'always' || o.showAdditionalFieldsForm === 'always') {
						result = false;
					} else {
						this.postAcceptedOffer(o);
						o.posted = true;
					}
				} else {
					result = false;
				}				
			} else {
				this.postAcceptedOffer(o);
				o.posted = true;
			}
		}
	}
	return result;

},
getStateList: function(countryCode, callback) {

	var url = PFLEX.AppInfo.getAppInfoPflxAdUrl() + "/global/state/" +countryCode.toLowerCase()+".json";
	var headers = PFLEX.VersionHelper.getHeaders();
	PFLEX.receiveMessage(callback, url, headers);
}


});
// *** signup, Registration, MissingInfo actions ***

PFLEX.SignUp = PFLEX.create();

PFLEX.SignUp.prototype = {
    init: function (offer, fields, signUpRequired, headerText) {
	}
};

PFLEX.extend(PFLEX.SignUp, {
	initialize : function(offer, fields, signUpRequired, headerText) {
		PFLEX.SignUp.config[0] = offer;
		PFLEX.SignUp.config[1] = fields;
		PFLEX.SignUp.config[2] = signUpRequired;
		PFLEX.SignUp.config[3] = headerText;
		PFLEX.SignUp.config[4] = "true";
		PFLEX.SignUp.config[5] = "registration"; //possible values are registration, missinginfo
	},
	config: {},
        showAdditionalFieldsForm: 'always',
	regulatoryMessagePrefix: 'regulatory_',
	getOffer: function() {
		return PFLEX.SignUp.config[0];
	},
	setOffer: function(offer) {
		PFLEX.SignUp.config[0] = offer;
	},
	getFields: function() {
		return PFLEX.SignUp.config[1];
	},
	setFields: function(fields) {
		PFLEX.SignUp.config[1] = fields;
	},
	isSignUpRequired: function() {
		return PFLEX.SignUp.config[2];
	},
	setSignUpRequired: function(signUpRequired) {
		PFLEX.SignUp.config[2] = signUpRequired;
	},
	getHeaderText: function() {
		return PFLEX.SignUp.config[3];
	},
	getField: function(fieldName) {
		var fieldsToConsider = PFLEX.SignUp.config[1];
		return fieldsToConsider[fieldName];
	},
	isShowDisplayMultiad: function() {
		return PFLEX.SignUp.config[4];
	},
	setShowDisplayMultiad: function(showDisplayMultiad) {
		PFLEX.SignUp.config[4] = showDisplayMultiad;
	},
	getPageType: function() {
		return PFLEX.SignUp.config[5];
	},
	setPageType: function(pageType) {
		PFLEX.SignUp.config[5] = pageType;
	}

});

PFLEX.SignUpField = PFLEX.create();

PFLEX.SignUpField.prototype = {
    init: function (fieldName, displayName, order, value, type, validations, styleSheet, 
					errorMessage, populatedFromDevice, optionType, transientField, options, markedForOfferRendering, fieldType) {
		this.fieldName = fieldName;
		this.displayName = displayName;
		this.order = order;
		this.value = value;
		this.type = type;
		this.validations = validations;
		this.styleSheet = styleSheet;
		this.errorMessage = errorMessage;
		this.populatedFromDevice = populatedFromDevice;
		this.optionType = optionType;
		this.transientField = transientField;
		this.options = options;
		this.markedForOfferRendering = markedForOfferRendering; //this to take care during rotation cases
		this.fieldType = fieldType;
	},
	setValue: function(value) {
		this.value = value;	
	},
	getValue: function() {
		return this.value;	
	},
	getValidations: function() {
		return this.validations;	
	},
	setErrorMessage: function(message) {
		this.errorMessage = message;
	},
	getErrorMessage: function() {
		return this.errorMessage;
	},
	clearErrorMessage: function() {
		this.errorMessage = null;;
	},
	isPopulatedFromDevice: function() {
		return this.populatedFromDevice;
	},
	setPopulatedFromDevice: function(populatedFromDevice) {
		this.populatedFromDevice = populatedFromDevice;
	},
	getOptionType: function() {
		return this.optionType;
	},
	isPhoneField: function() {
		var result = false;
		PFLEX.$A(this.validations).pflexEach(function(value){
			var validationName = ''+value.name;		
			if(validationName === 'usphone' || validationName === 'phone') {
				result = true;
			}
		});
		return result;
	}
};


PFLEX.SignupActions = PFLEX.create();

PFLEX.SignupActions.prototype = {

init: function () {
}
};

PFLEX.extend(PFLEX.SignupActions, {

leadsUrl: function () {
	return "https://app.pontiflex.com/bpost";
},
statsUrl: function () {
	return "http://stats.pflexads.com";
},
renderPlacement: function() {
	var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
	var signUpPageType = PFLEX.SignUp.getPageType();
	var signUpRequired = PFLEX.SignUp.isSignUpRequired();
	var renderPlacement = PFLEX.renderJson.Placement(placement, signUpPageType == null ? 'registration' : signUpPageType, document.getElementById('main'), signUpRequired);		
},
loadFields: function () {
        var result = {};
	var fieldNameJson = PFLEX.AppInfo.getRegistrationFields();
	for( var key in fieldNameJson ) {
		var fieldJson = JSON.parse(JSON.stringify(fieldNameJson[key]));
		var validations = [];
		var validationsJosn = JSON.parse(JSON.stringify(fieldJson.validations));
		var counter = 0;		
		for( var keyValidation in validationsJosn) {
			validations.push(PFLEX.Validation.get(validationsJosn[counter]));
			counter = counter + 1;
		}

		var field = new PFLEX.SignUpField(key, fieldJson.displayName, fieldJson.order, '', fieldJson.type, validations, '', '', "false", 
												fieldJson.optionType, fieldJson.transientField, fieldJson.options, false, fieldJson.fieldType);
        result[key] = field;		
	}
	return result;
	
},
populateFields: function () {
	var fields = PFLEX.AppInfo.getOrderedFields();
	var valueFromStateStorgaeJSON = PFLEX.StorageUtil.getRegFormState();

	for( var i=0; i<fields.length; i++ ) {
		var key = fields[i].fieldName;
		var signupField = PFLEX.SignUp.getField(key);
		var valueFromStorage = PFLEX.StorageUtil.getRegistrationData(key);
		
		if(typeof valueFromStorage !== "undefined") {
			signupField.setValue(''+valueFromStorage);
		}
		if(typeof valueFromStateStorgaeJSON !== "undefined") {
                        if(valueFromStateStorgaeJSON !== null) {
				var valueFromStateStorgae = valueFromStateStorgaeJSON[key];
				if(typeof valueFromStateStorgae !== "undefined") {
					signupField.setValue(''+valueFromStateStorgae.value);
				}
				signupField.setErrorMessage(''+valueFromStateStorgae.errorMessage);
				signupField.setPopulatedFromDevice(''+valueFromStateStorgae.populatedFromDevice);
			}
		}

		//if field value is still empty, try to populate it from device
		var signupFieldValue = signupField.getValue();
		if(typeof signupFieldValue === "undefined" || signupFieldValue === "") {
			var valueFromDevice = ''+PFLEX.NativeDeviceData.getDeviceData(key);
			if(typeof valueFromDevice !== "undefined" && valueFromDevice !== null && valueFromDevice !== "") {
				signupField.setValue(''+valueFromDevice);
				if(key === "email") {
					signupField.setPopulatedFromDevice("true");
				}
			}
		}
	}

	//if we don't have saved registration state then set to track impressions also
		//track offers loaded and application impression
	//if its missinginfo, don't track it
	if(typeof valueFromStateStorgaeJSON === "undefined" || valueFromStateStorgaeJSON === null) {
		var registrationOffer = PFLEX.AppInfo.getOffer();
		var pageType = PFLEX.SignUp.getPageType();
		if(typeof pageType !== "undefined" && pageType === "registration") {
			PFLEX.SignupActions.trackAppImpression();
			PFLEX.SignupActions.trackImpression(registrationOffer);
		}
	}



	PFLEX.StateStorage.clearRegFormState();
},
decline: function() {
	PFLEX.Native.finishActivity();
},
finishActivity: function () {
	PFLEX.Native.finishActivity();
},
initializeData: function () {
	PFLEX.Native.initializeData();
},
saveRegistrationData: function () {
	var fields = PFLEX.AppInfo.getRegistrationFields();
	var postal_code_fieldName = PFLEX.AppInfo.getPostalCodeFieldName();
	var country_fieldName = PFLEX.AppInfo.getCountryFieldName();
	for( var key in fields ) {
		var signupField = PFLEX.SignUp.getField(key);
		var valueFromSignupField = signupField.getValue();
		
		if(postal_code_fieldName != null && key == postal_code_fieldName) {
			var countryField = PFLEX.SignUp.getField(country_fieldName);
			PFLEX.StorageUtil.addRegistrationData(country_fieldName, ''+countryField.getValue());
		}
		if(signupField.isPhoneField()) {
			var phoneDate = PFLEX.PhoneUtil.parsePhone(signupField.value);
			signupField.value = PFLEX.PhoneUtil.format(phoneDate);
			valueFromSignupField = signupField.getValue();
		}
		PFLEX.StorageUtil.addRegistrationData(key, ''+valueFromSignupField);
		
	}	
},
postAcceptedOffer: function (offer) {
	PFLEX.BaseActions.postAcceptedOffer(offer);	
},
trackAppImpression: function () {
	PFLEX.BaseActions.trackAppImpression();
},

trackImpressions: function () {
	PFLEX.BaseActions.trackImpressions();
},

trackImpression: function (offer) {
	PFLEX.BaseActions.trackImpression(offer);
},

resolvePageType: function () {
	var pageType = PFLEX.PageTransitionState.getSignupPageType();
	if(typeof pageType !== "undefined" && pageType !== null) {
		PFLEX.SignUp.setPageType(''+pageType);
	}
	PFLEX.PageTransitionState.clearSignupPageType();
},
confirm: function() {
	var pageType = PFLEX.SignUp.getPageType();
	if(typeof pageType !== "undefined" && pageType !== null) {
		if(pageType === "registration") {
			PFLEX.RegistrationActions.confirm();
		}
		if(pageType === "missinginfo") {
			PFLEX.MissigninfoActions.confirm();
		}		
	}
},
thanksAndFinish: function() {	
	PFLEX.BaseActions.thanksAndFinish();
}


});

PFLEX.RegistrationActions = PFLEX.create();

PFLEX.RegistrationActions.prototype = {
};

PFLEX.extend(PFLEX.RegistrationActions, {

confirm: function() {
	PFLEX.SignupActions.saveRegistrationData();
	PFLEX.StorageUtil.setRegLastVerifiedDate(PFLEX.DateUtil.formatAsTimestamp(PFLEX.DateUtil.getUTCNow()));
	PFLEX.SignupActions.postAcceptedOffer(PFLEX.SignUp.getOffer());
	var isOnlineValue = ''+PFLEX.Native.isOnline();
	if(PFLEX.SignUp.isShowDisplayMultiad() === "true" && isOnlineValue === "true") {
		//load multioffer page
		location.replace('./multioffer.html');
	} else {
		PFLEX.SignupActions.thanksAndFinish();		
	}
	
}

});

PFLEX.MissigninfoActions = PFLEX.create();

PFLEX.MissigninfoActions.prototype = {
};

PFLEX.extend(PFLEX.MissigninfoActions, {

confirm: function() {
	PFLEX.SignupActions.saveRegistrationData();
	PFLEX.StorageUtil.setRegLastVerifiedDate(PFLEX.DateUtil.formatAsTimestamp(PFLEX.DateUtil.getUTCNow()));
	var postingResult = PFLEX.MissigninfoActions.postAcceptedOffers();

        //move to offer signup page
	if(postingResult) {
		PFLEX.PageTransitionState.clearOffersForMissingInfoPage();
		PFLEX.SignupActions.thanksAndFinish();
	} else {
		PFLEX.MissigninfoActions.saveInfoForOfferSignupPage();					
		PFLEX.PageTransitionState.clearOffersForMissingInfoPage();
		location.replace('./offer_signup.html');			
	}		
},
saveInfoForOfferSignupPage: function() {
		//set variables for offer signup page
		var offersList = PFLEX.PageTransitionState.getOffersForMissingInfoPage();
		PFLEX.PageTransitionState.setOffersForOfferSignupPage(offersList);

},
postAcceptedOffers: function() {
	var result = true;
	var offersList = PFLEX.PageTransitionState.getOffersForMissingInfoPage();	
	return 	PFLEX.BaseActions.postAcceptedOffers(offersList);

	//PFLEX.PageTransitionState.clearOffersForMissingInfoPage();
}

});
// *** Footer ***

// *** START ***
}
// *** END ***

})();
// *** Post Message Header header.js***

(function($ns){

// *** Post Message post-message.js***

/* Using AJAX calls for e.g., to load offers, load images from ad server */

var savedCallback;


$ns.receiveMessage = savedReceiveMessage = function(callback, targetUrl, headers) {
   var request = false;
   try {
     request = new XMLHttpRequest();
   } catch (trymicrosoft) {
     try {
       request = new ActiveXObject("Msxml2.XMLHTTP");
     } catch (othermicrosoft) {
       try {
         request = new ActiveXObject("Microsoft.XMLHTTP");
       } catch (failed) {
         request = false;
       }  
     }
   }


   if (!request) {
     PFLEX.NativeActions.alert("Error initializing XMLHttpRequest!");        
   }

   request.open("GET", targetUrl, true);
   if (callback) {
	// Bind the callback.
            savedCallback = function() {		
		if (request.readyState == 4) {
		       if (request.status == 200) {
				clearTimeout(xmlHttpTimeout); 
				var response = request.responseText;
                		callback(''+response);
			}
		}
            };		
   	request.onreadystatechange = savedCallback;
   }

   PFLEX.console.log('post message targetUrl='+targetUrl);

   if(typeof headers !== "undefined") {
   	for(var key in headers) {
		request.setRequestHeader(''+key, ''+headers[key]);
   	}
   }

   request.send(null);
   // Timeout to abort in 30 seconds
   var xmlHttpTimeout=setTimeout(ajaxTimeout,30000);
   function ajaxTimeout(){
   	request.abort();
   	console.log("Pontiflex SDK request timed out after 30 seconds");
	callback();
   }

};
// *** Post Message Footer ***

})(PFLEX);

// *** Init ***

(function() {
	//set classes like portrait/landscape, tablet
	PFLEX.BaseActions.setScreenOrientationAndSize();

	//initialize update resource manager
	PFLEX.UpdateResource.Manager.initResourcesFromConfig();
	
	
var initPage = function(jsonstr){

	PFLEX.locale = PFLEX.Native.getLocale();

	PFLEX.AppInfo.initialize(jsonstr);
	
	//make call to update resource manager
	PFLEX.UpdateResource.Manager.updateResources();
	
	var fields = PFLEX.SignupActions.loadFields();
	var registrationOffer = PFLEX.AppInfo.getOffer();
	PFLEX.SignUp.setFields(fields);
	PFLEX.SignUp.setOffer(registrationOffer);


	PFLEX.SignupActions.resolvePageType();
	PFLEX.SignupActions.populateFields();

	//render the fields
	PFLEX.SignupActions.renderPlacement();
	PFLEX.Native.cancelProgressDialog();

};
	
	var loadAppInfo = function(){
		var appinfojsonContent = PFLEX.ResourceUtil.getResource('appinfojson');
		if(appinfojsonContent === null || typeof appinfojsonContent === undefined) {
			console.log("Unable to load appinfo resource");
		}
		initPage(appinfojsonContent);
	};
	
	
	PFLEX.AppInfo.eventHandler = new PFLEX.EventHandler();
	 
	PFLEX.AppInfo.eventHandler.register("onSetConfig",function(eventName,stringFromReg,unsure,objFromFire){ 
		PFLEX.SignUp.initialize(); 
		PFLEX.SignUp.setSignUpRequired(''+objFromFire.signUpRequired);
		PFLEX.SignUp.setShowDisplayMultiad(''+objFromFire.showMultiAd);
		loadAppInfo(); 
	});

	

	PFLEX.BaseActions.resolveCurrentPage('registration.html');
	var currentPage = PFLEX.BaseActions.getCurrentPage();
	if(typeof currentPage !== "undefined" && ''+currentPage === 'registration.html') {
		PFLEX.Native.initializeData();	
	}

})();

